@@ -680,21 +680,22 @@ def _lessbroken_deepcopy(a):
     # This logic is also in link.py
     from aesara.graph.type import _cdata_type
 
-    if type(a) in (np.ndarray, np.memmap):
+    if isinstance(a, (np.ndarray, np.memmap)):
         rval = a.copy(order="K")
-    elif type(a) is _cdata_type:
+    elif isinstance(a, _cdata_type):
         # This is not copyable (and should be used for constant data).
         rval = a
     else:
         rval = copy.deepcopy(a)
 
     assert type(rval) == type(a), (type(rval), type(a))
+
     if isinstance(rval, np.ndarray):
         assert rval.dtype == a.dtype
     return rval
 
 
-def _find_bad_optimizations0(order, reasons, r_vals):
+def _find_bad_optimizations(order, reasons, r_vals):
     """
     Use a simple algorithm to find broken optimizations.
 
@@ -714,7 +715,7 @@ def _find_bad_optimizations0(order, reasons, r_vals):
                 # check if the value for new_r doesn't match the value for r
                 new_r_val = r_vals[new_r]
                 r_val = r_vals[r]
-                assert r.type == new_r.type
+                assert r.type.is_super(new_r.type)
 
                 if hasattr(new_r.tag, "values_eq_approx"):
                     check = new_r.tag.values_eq_approx(r_val, new_r_val)
@@ -735,119 +736,6 @@ def _find_bad_optimizations0(order, reasons, r_vals):
                     )
 
 
-def _find_bad_optimizations1(order, reasons, r_vals):
-    # iterate over variables looking for values that don't match the
-    # values of the variables they replaced.  This is the sign of a
-    # broken optimization.
-
-    # identify sets of variables that are supposed to be equivalent
-    equivalence_sets = {}
-    program_position = {}  # node -> order idx
-
-    for i, node in enumerate(order):
-        program_position[node] = i
-        for new_r in node.outputs:
-            equivalence_sets.setdefault(new_r, {new_r})
-            for reason, r, old_graph_str, new_graph_str in reasons[new_r]:
-                equivalence_sets[new_r].update(equivalence_sets.setdefault(r, {r}))
-                for er in equivalence_sets[r]:
-                    equivalence_sets[er] = equivalence_sets[new_r]
-
-    # identify equivalence sets that are broken
-    equivalence_sets_broken = {}  # id(set) -> Bool
-    there_is_a_problem = False
-    for r, r_equiv in equivalence_sets.items():
-        if id(r_equiv) not in equivalence_sets_broken:
-            equivalence_sets_broken[id(r_equiv)] = False
-            # loop over the variables in the set comparing them to be
-            # equal enough
-            re0 = None
-            for re in r_equiv:
-                if re0:
-                    new_r_val = r_vals[re]
-                    r_val = r_vals[re0]
-                    assert re.type == re0.type
-                    if not re.type.values_eq_approx(r_val, new_r_val):
-                        equivalence_sets_broken[id(r_equiv)] = True
-                        there_is_a_problem = True
-                re0 = re
-
-    if there_is_a_problem:
-        # which broken equivalence set has the earliest-occurring element?
-        first_broken_set = None
-        for i, node in enumerate(order):
-            for r in node.outputs:
-                r_equiv = equivalence_sets[r]
-                if equivalence_sets_broken[id(r_equiv)]:
-                    first_broken_set = r_equiv
-        # TODO finish this to produce good diagnostic information
-        print(first_broken_set)
-        raise Exception("broken")
-
-
-def _find_bad_optimizations2(order, reasons, r_vals):
-    """
-    Use a simple algorithm to find broken optimizations.
-
-    This algorithm is simple to understand, but sometimes when there's
-    a problem it identifies the wrong optimization as the culprit.
-    The problem stems from the fact that results are not evaluated in
-    chronological order (looking at when they were introduced to the
-    graph).
-
-    """
-
-    checked_variables = set()
-
-    def check_variable_norec(new_r):
-        """
-        Verify that `r` has the same value as the results it replaces.
-
-        """
-        for reason, r, old_graph_str, new_graph_str in reasons[new_r]:
-            new_r_val = r_vals[new_r]
-            r_val = r_vals[r]
-
-            if (r.type != new_r.type) or (
-                not r.type.values_eq_approx(r_val, new_r_val)
-            ):
-                raise BadOptimization(
-                    old_r=r,
-                    new_r=new_r,
-                    old_r_val=r_val,
-                    new_r_val=new_r_val,
-                    reason=reason,
-                    old_graph=old_graph_str,
-                    new_graph=new_graph_str,
-                )
-
-    def check_variable(r):
-        if r in checked_variables:
-            return
-        checked_variables.add(r)
-
-        # (recursively) first check all the variables that could make
-        # r look bad:
-        list_of_vars = [old_r for (reason, old_r, olds, news) in reasons[r]]
-        if None is not r.owner:
-            list_of_vars += r.owner.inputs
-
-        for var_that_could_make_r_look_bad in list_of_vars:
-            check_variable(var_that_could_make_r_look_bad)
-
-        check_variable_norec(r)
-
-    # iterate over variables looking for values that don't match the
-    # values of the variables they replaced.  This is the sign of a
-    # broken optimization.
-    for i, node in enumerate(order):
-        for new_r in node.outputs:
-            check_variable(new_r)
-
-
-_find_bad_optimizations = _find_bad_optimizations0
-
-
 def _get_preallocated_maps(
     node,
     thunk,
@@ -1173,7 +1061,7 @@ def _check_preallocated_output(
                         r,
                         storage_map[r][0],
                         hint=thunk_name,
-                        specific_hint=r.type.value_validity_msg(storage_map[r][0]),
+                        specific_hint=validity_hint(r.type, storage_map[r][0]),
                     )
 
             _check_inputs(
@@ -1213,6 +1101,14 @@ def _check_preallocated_output(
             fn.maker.mode = backup_mode
 
 
+def validity_hint(type, x):
+    try:
+        type.filter(x, strict=True)
+    except Exception as e:
+        return str(e)
+    return "value is valid"
+
+
 class _FunctionGraphEvent:
     """
     A record of an event in the life of an FunctionGraph.
@@ -1277,7 +1173,7 @@ class _FunctionGraphEvent:
             # nodes are not compared because this comparison is
             # supposed to be true for corresponding events that happen
             # in different FunctionGraph instances (different graphs)
-            for attr in ["kind", "op", "idx", "reason"]:
+            for attr in ("kind", "op", "idx", "reason"):
                 rval = rval and getattr(self, attr) == getattr(other, attr)
         return rval
 
@@ -1755,7 +1651,7 @@ class _Linker(LocalLinker):
                         # check output values for type-correctness
                         for r in node.outputs:
                             if not r.type.is_valid_value(storage_map[r][0]):
-                                hint2 = r.type.value_validity_msg(storage_map[r][0])
+                                hint2 = validity_hint(r.type, storage_map[r][0])
                                 raise InvalidValueError(
                                     r,
                                     storage_map[r][0],
@@ -2006,7 +1902,7 @@ class _Linker(LocalLinker):
                         # HACK TO LOOK LIKE A REAL DESTRUCTIVE ACTION
                         # TOOK PLACE
                         if (
-                            (type(dr_vals[r][0]) in (np.ndarray, np.memmap))
+                            isinstance(dr_vals[r][0], (np.ndarray, np.memmap))
                             and (dr_vals[r][0].dtype == storage_map[r][0].dtype)
                             and (dr_vals[r][0].shape == storage_map[r][0].shape)
                         ):
