@@ -249,7 +249,7 @@ class Apply(Node):
         remake_node = False
         new_inputs = inputs[:]
         for i, (curr, new) in enumerate(zip(self.inputs, new_inputs)):
-            if not curr.type == new.type:
+            if curr.type != new.type:
                 if strict:
                     # If compatible, casts new into curr.type
                     new_inputs[i] = curr.type.filter_variable(new)
@@ -358,10 +358,10 @@ class Variable(Node):
     .. code-block:: python
 
         import aesara
-        import aesara.tensor as aet
+        import aesara.tensor as at
 
-        a = aet.constant(1.5)            # declare a symbolic constant
-        b = aet.fscalar()                # declare a symbolic floating-point scalar
+        a = at.constant(1.5)            # declare a symbolic constant
+        b = at.fscalar()                # declare a symbolic floating-point scalar
 
         c = a + b                       # create a simple expression
 
@@ -515,9 +515,9 @@ class Variable(Node):
         --------
 
         >>> import numpy as np
-        >>> import aesara.tensor as aet
-        >>> x = aet.dscalar('x')
-        >>> y = aet.dscalar('y')
+        >>> import aesara.tensor as at
+        >>> x = at.dscalar('x')
+        >>> y = at.dscalar('y')
         >>> z = x + y
         >>> np.allclose(z.eval({x : 16.3, y : 12.1}), 28.4)
         True
@@ -1154,7 +1154,7 @@ def io_toposort(
             # We suppose that all outputs are always computed
             if cur.outputs[0] in computed:
                 continue
-            if all([i in computed or i.owner is None for i in cur.inputs]):
+            if all(i in computed or i.owner is None for i in cur.inputs):
                 computed.update(cur.outputs)
                 order.append(cur)
             else:
@@ -1444,7 +1444,7 @@ def is_in_ancestors(l_apply: Apply, f_node: Apply) -> bool:
         cur = todo.pop()
         if cur.outputs[0] in computed:
             continue
-        if all([i in computed or i.owner is None for i in cur.inputs]):
+        if all(i in computed or i.owner is None for i in cur.inputs):
             computed.update(cur.outputs)
             if cur is f_node:
                 return True
@@ -1543,14 +1543,14 @@ def equal_computations(xs, ys, in_xs=None, in_ys=None):
         if x.owner:  # Check above tell that y.owner eval to True too.
             if x.owner.outputs.index(x) != y.owner.outputs.index(y):
                 return False
-        if x not in in_xs and x.type != y.type:
+        if x not in in_xs and not (y.type.in_same_class(x.type)):
             return False
 
     if len(in_xs) != len(in_ys):
         return False
 
     for _x, _y in zip(in_xs, in_ys):
-        if _x.type != _y.type:
+        if not (_y.type.in_same_class(_x.type)):
             return False
 
     common = set(zip(in_xs, in_ys))
@@ -1648,3 +1648,40 @@ def equal_computations(xs, ys, in_xs=None, in_ys=None):
                 return False
 
     return True
+
+
+def get_var_by_name(
+    graphs: Iterable[Variable], target_var_id: str, ids: str = "CHAR"
+) -> Tuple[Variable]:
+    r"""Get variables in a graph using their names.
+
+    Parameters
+    ----------
+    graphs:
+        The graph, or graphs, to search.
+    target_var_id:
+        The name to match against either ``Variable.name`` or
+        ``Variable.auto_name``.
+
+    Returns
+    -------
+    A ``tuple`` containing all the `Variable`\s that match `target_var_id`.
+
+    """
+    from aesara.graph.op import HasInnerGraph
+
+    def expand(r):
+        if r.owner:
+            res = r.owner.inputs
+
+            if isinstance(r.owner.op, HasInnerGraph):
+                res.extend(r.owner.op.inner_outputs)
+
+            return res
+
+    results = ()
+    for var in walk(graphs, expand, False):
+        if target_var_id == var.name or target_var_id == var.auto_name:
+            results += (var,)
+
+    return results
