@@ -110,7 +110,7 @@ class BadOptimization(Exception):
         used_ids = dict()
 
         if isinstance(old_r, Variable):
-            self.old_graph = aesara.compile.debugmode.debugprint(
+            self.old_graph = aesara.printing._debugprint(
                 old_r,
                 prefix="  ",
                 depth=6,
@@ -123,7 +123,7 @@ class BadOptimization(Exception):
             self.old_graph = None
 
         if isinstance(new_r, Variable):
-            self.new_graph = aesara.compile.debugmode.debugprint(
+            self.new_graph = aesara.printing._debugprint(
                 new_r,
                 prefix="  ",
                 depth=6,
@@ -366,8 +366,8 @@ class LambdaExtract:
         self.reason = reason
 
     def __call__(self):
-        return self.fgraph.change_input(
-            self.node, self.i, self.r, reason=("Revert", self.reason)
+        return self.fgraph.change_node_input(
+            self.node, self.i, self.r, reason=("Revert", self.reason), check=False
         )
 
 
@@ -553,12 +553,9 @@ class ReplaceValidate(History, Validator):
         self, fgraph, replacements, reason=None, verbose=None, **kwargs
     ):
         chk = fgraph.checkpoint()
+
         if verbose is None:
             verbose = config.optimizer_verbose
-        if config.scan__debug:
-            from aesara.scan.op import Scan
-
-            scans = [n for n in fgraph.apply_nodes if isinstance(n.op, Scan)]
 
         for r, new_r in replacements:
             try:
@@ -596,29 +593,14 @@ class ReplaceValidate(History, Validator):
         except Exception as e:
             fgraph.revert(chk)
             if verbose:
-                print(f"validate failed on node {r}.\n Reason: {reason}, {e}")
+                print(
+                    f"optimizer: validate failed on node {r}.\n Reason: {reason}, {e}"
+                )
             raise
-        if config.scan__debug:
-            from aesara.scan.op import Scan
 
-            scans2 = [n for n in fgraph.apply_nodes if isinstance(n.op, Scan)]
-            nb = len(scans)
-            nb2 = len(scans2)
-            if nb2 > nb:
-                print(
-                    "Extra scan introduced",
-                    nb,
-                    nb2,
-                    getattr(reason, "name", reason),
-                    r,
-                    new_r,
-                )
-            elif nb2 < nb:
-                print(
-                    "Scan removed", nb, nb2, getattr(reason, "name", reason), r, new_r
-                )
         if verbose:
-            print(reason, r, new_r)
+            print(f"optimizer: rewrite {reason} replaces {r} with {new_r}")
+
         # The return is needed by replace_all_validate_remove
         return chk
 
@@ -783,6 +765,8 @@ class PreserveVariableAttributes(Feature):
 
 
 class NoOutputFromInplace(Feature):
+    """Prevent `FunctionGraph` outputs within a range from being altered in-place."""
+
     def __init__(self, first_output_idx=0, last_output_idx=None):
         self.first_idx = first_output_idx
         self.last_idx = last_output_idx
@@ -791,19 +775,18 @@ class NoOutputFromInplace(Feature):
         if not hasattr(fgraph, "destroyers"):
             return True
 
-        outputs_to_validate = list(fgraph.outputs)[self.first_idx : self.last_idx]
+        for out in fgraph.outputs[self.first_idx : self.last_idx]:
 
-        for out in outputs_to_validate:
+            node = out.owner
 
-            if out.owner is None:
+            if node is None:
                 continue
 
             # Validate that the node that produces the output does not produce
-            # it by modifying something else inplace.
-            node = out.owner
+            # it by modifying something else in-place.
             op = node.op
             out_idx = node.outputs.index(out)
-            if op.destroy_map and out_idx in op.destroy_map:
+            if out_idx in op.destroy_map:
                 raise aesara.graph.fg.InconsistencyError(
                     "A function graph Feature has requested that outputs of the graph "
                     "be prevented from being the result of in-place "
