@@ -418,18 +418,20 @@ class FunctionGraph(MetaObject):
                 self.add_client(input, (node, i))
             self.execute_callbacks("on_import", node, reason)
 
-    def change_input(
+    def change_node_input(
         self,
         node: Union[Apply, str],
         i: int,
         new_var: Variable,
         reason: str = None,
         import_missing: bool = False,
+        check: bool = True,
     ) -> None:
         """Change ``node.inputs[i]`` to `new_var`.
 
-        ``new_var.type == old_var.type`` must be ``True``, where ``old_var`` is the
-        current value of ``node.inputs[i]`` which we want to replace.
+        ``new_var.type.is_super(old_var.type)`` must be ``True``, where
+        ``old_var`` is the current value of ``node.inputs[i]`` which we want to
+        replace.
 
         For each feature that has an `on_change_input` method, this method calls:
         ``feature.on_change_input(function_graph, node, i, old_var, new_var, reason)``
@@ -446,26 +448,27 @@ class FunctionGraph(MetaObject):
             The new variable to take the place of ``node.inputs[i]``.
         import_missing
             Add missing inputs instead of raising an exception.
+        check
+            When ``True``, perform a type check between the variable being
+            replaced and its replacement.  This is primarily used by the
+            `History` `Feature`, which needs to revert types that have been
+            narrowed and would otherwise fail this check.
         """
         # TODO: ERROR HANDLING FOR LISTENERS (should it complete the change or revert it?)
         if node == "output":
             r = self.outputs[i]
-            if not r.type == new_var.type:
+            if check and not r.type.is_super(new_var.type):
                 raise TypeError(
-                    "The type of the replacement must be the"
-                    " same as the type of the original Variable.",
-                    r,
-                    new_var,
+                    f"The type of the replacement ({new_var.type}) must be "
+                    f"compatible with the type of the original Variable ({r.type})."
                 )
             self.outputs[i] = new_var
         else:
             r = node.inputs[i]
-            if not r.type == new_var.type:
+            if check and not r.type.is_super(new_var.type):
                 raise TypeError(
-                    "The type of the replacement must be the"
-                    " same as the type of the original Variable.",
-                    r,
-                    new_var,
+                    f"The type of the replacement ({new_var.type}) must be "
+                    f"compatible with the type of the original Variable ({r.type})."
                 )
             node.inputs[i] = new_var
 
@@ -544,7 +547,7 @@ class FunctionGraph(MetaObject):
             assert (node == "output" and self.outputs[i] is var) or (
                 node.inputs[i] is var
             )
-            self.change_input(
+            self.change_node_input(
                 node, i, new_var, reason=reason, import_missing=import_missing
             )
 
