@@ -13,6 +13,7 @@ import sys
 import warnings
 from abc import abstractmethod
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
     ClassVar,
@@ -43,6 +44,9 @@ from aesara.graph.utils import (
 from aesara.link.c.interface import CLinkerOp
 
 
+if TYPE_CHECKING:
+    from aesara.compile.function.types import Function
+
 StorageMapType = List[Optional[List[Any]]]
 ComputeMapType = List[bool]
 OutputStorageType = List[Optional[List[Any]]]
@@ -219,9 +223,17 @@ class Op(MetaObject):
             raise ValueError(
                 f"We expected {len(self.itypes)} inputs but got {len(inputs)}."
             )
-        if not all(inp.type == it for inp, it in zip(inputs, self.itypes)):
+        if not all(it.in_same_class(inp.type) for inp, it in zip(inputs, self.itypes)):
             raise TypeError(
-                f"We expected inputs of types '{str(self.itypes)}' but got types '{str([inp.type for inp in inputs])}'"
+                f"Invalid input types for Op {self}:\n"
+                + "\n".join(
+                    f"Input {i}/{len(inputs)}: Expected {inp}, got {out}"
+                    for i, (inp, out) in enumerate(
+                        zip(self.itypes, (inp.type for inp in inputs)),
+                        start=1,
+                    )
+                    if inp != out
+                )
             )
         return Apply(self, inputs, [o() for o in self.otypes])
 
@@ -574,6 +586,25 @@ class Op(MetaObject):
         return getattr(type(self), "__name__", super().__str__())
 
 
+class HasInnerGraph:
+    r"""A mixin for an `Op` that contain an inner graph."""
+
+    @property
+    @abstractmethod
+    def fn(self) -> "Function":
+        """The inner function."""
+
+    @property
+    @abstractmethod
+    def inner_inputs(self) -> List[Variable]:
+        """The inner function's inputs."""
+
+    @property
+    @abstractmethod
+    def inner_outputs(self) -> List[Variable]:
+        """The inner function's outputs."""
+
+
 class COp(Op, CLinkerOp):
     """An `Op` with a C implementation."""
 
@@ -712,7 +743,7 @@ def missing_test_message(msg: Text) -> None:
     elif action == "warn":
         warnings.warn(msg, stacklevel=2)
     else:
-        assert action in ["ignore", "off"]
+        assert action in ("ignore", "off")
 
 
 def get_test_values(*args: Variable) -> Union[Any, List[Any]]:
@@ -767,22 +798,6 @@ def get_test_values(*args: Variable) -> Union[Any, List[Any]]:
     return [tuple(rval)]
 
 
-ops_with_inner_function: Dict[Op, Text] = {}
-r"""
-Registry of `Op`\s that have an inner compiled Aesara function.
-
-The keys are `Op` classes (not instances), and values are the name of the
-attribute that contains the function. For instance, if the function is
-``self.fn``, the value will be ``'fn'``.
-
-We need that to be able not to run debug checks a number of times that is
-exponential in the nesting level of those `Op`\s.
-
-For instance, `Scan` will be registered here.
-
-"""
-
-
 class OpenMPOp(COp):
     r"""Base class for `Op`\s using OpenMP.
 
