@@ -13,10 +13,11 @@ is a global operation with a scalar condition.
 
 import logging
 from copy import deepcopy
+from typing import List, Union
 
 import numpy as np
 
-import aesara.tensor as aet
+import aesara.tensor as at
 from aesara.compile import optdb
 from aesara.configdefaults import config
 from aesara.graph.basic import Apply, Variable, clone_replace, is_in_ancestors
@@ -24,7 +25,6 @@ from aesara.graph.op import _NoPythonOp
 from aesara.graph.opt import GlobalOptimizer, in2out, local_optimizer
 from aesara.tensor import basic
 from aesara.tensor.shape import Reshape, Shape, SpecifyShape
-from aesara.tensor.type import TensorType
 
 
 __docformat__ = "restructedtext en"
@@ -62,13 +62,16 @@ class IfElse(_NoPythonOp):
         ``rval = ifelse(condition, rval_if_true1, .., rval_if_trueN,
                         rval_if_false1, rval_if_false2, .., rval_if_falseN)``
 
-    :note:
+    .. note:
+
         Other Linkers then CVM and VM are INCOMPATIBLE with this Op, and
         will ignore its lazy characteristic, computing both the True and
         False branch before picking one.
 
     """
 
+    __props__ = ("as_view", "gpu", "n_outs")
+
     def __init__(self, n_outs, as_view=False, gpu=False, name=None):
         if as_view:
             # check destroyhandler and others to ensure that a view_map with
@@ -83,21 +86,18 @@ class IfElse(_NoPythonOp):
         self.name = name
 
     def __eq__(self, other):
-        if not type(self) == type(other):
+        if type(self) != type(other):
             return False
-        if not self.as_view == other.as_view:
+        if self.as_view != other.as_view:
             return False
-        if not self.gpu == other.gpu:
+        if self.gpu != other.gpu:
             return False
-        if not self.n_outs == other.n_outs:
+        if self.n_outs != other.n_outs:
             return False
         return True
 
     def __hash__(self):
-        rval = (
-            hash(type(self)) ^ hash(self.as_view) ^ hash(self.gpu) ^ hash(self.n_outs)
-        )
-        return rval
+        return hash((type(self), self.as_view, self.gpu, self.n_outs))
 
     def __str__(self):
         args = []
@@ -171,7 +171,7 @@ class IfElse(_NoPythonOp):
                 f"Wrong number of arguments to make_node: expected "
                 f"{int(2 * self.n_outs)}, got {len(args)}"
             )
-        c = aet.basic.as_tensor_variable(c)
+        c = at.basic.as_tensor_variable(c)
         if not self.gpu:
             # When gpu is true, we are given only gpuarrays, and we want
             # to keep them as gpuarrays
@@ -180,7 +180,7 @@ class IfElse(_NoPythonOp):
                 if isinstance(x, Variable):
                     nw_args.append(x)
                 else:
-                    nw_args.append(aet.as_tensor_variable(x))
+                    nw_args.append(at.as_tensor_variable(x))
             args = nw_args
         aes = args[: self.n_outs]
         fs = args[self.n_outs :]
@@ -189,9 +189,9 @@ class IfElse(_NoPythonOp):
             # TODO: Attempt to convert types so that they match?
             # new_f = t.type.filter_variable(f)
 
-            if t.type != f.type:
+            if not t.type.is_super(f.type):
                 raise TypeError(
-                    "IfElse requires same types for true and false return values: "
+                    "IfElse requires compatible types for true and false return values: "
                     f"true_branch={t.type}, false_branch={f.type}"
                 )
         if c.ndim > 0:
@@ -229,11 +229,11 @@ class IfElse(_NoPythonOp):
         if_true = (
             [ins[0]]
             + grads
-            + [aet.basic.zeros_like(t, dtype=grads[i].dtype) for i, t in enumerate(aes)]
+            + [at.basic.zeros_like(t, dtype=grads[i].dtype) for i, t in enumerate(aes)]
         )
         if_false = (
             [ins[0]]
-            + [aet.basic.zeros_like(f, dtype=grads[i].dtype) for i, f in enumerate(fs)]
+            + [at.basic.zeros_like(f, dtype=grads[i].dtype) for i, f in enumerate(fs)]
             + grads
         )
 
@@ -274,7 +274,7 @@ class IfElse(_NoPythonOp):
                             if self.as_view:
                                 storage_map[out][0] = val
                             # Work around broken numpy deepcopy
-                            elif type(val) in (np.ndarray, np.memmap):
+                            elif isinstance(val, (np.ndarray, np.memmap)):
                                 storage_map[out][0] = val.copy()
                             else:
                                 storage_map[out][0] = deepcopy(val)
@@ -294,7 +294,7 @@ class IfElse(_NoPythonOp):
                             # improves
                             # Work around broken numpy deepcopy
                             val = storage_map[f][0]
-                            if type(val) in (np.ndarray, np.memmap):
+                            if isinstance(val, (np.ndarray, np.memmap)):
                                 storage_map[out][0] = val.copy()
                             else:
                                 storage_map[out][0] = deepcopy(val)
@@ -306,35 +306,40 @@ class IfElse(_NoPythonOp):
         return thunk
 
 
-def ifelse(condition, then_branch, else_branch, name=None):
+def ifelse(
+    condition: Variable,
+    then_branch: Union[Variable, List[Variable]],
+    else_branch: Union[Variable, List[Variable]],
+    name: str = None,
+) -> Union[Variable, List[Variable]]:
     """
     This function corresponds to an if statement, returning (and evaluating)
     inputs in the ``then_branch`` if ``condition`` evaluates to True or
     inputs in the ``else_branch`` if ``condition`` evaluates to False.
 
-    :type condition: scalar like
-    :param condition:
+    Parameters
+    ==========
+    condition
         ``condition`` should be a tensor scalar representing the condition.
         If it evaluates to 0 it corresponds to False, anything else stands
         for True.
 
-    :type then_branch: list of aesara expressions/ aesara expression
-    :param then_branch:
+    then_branch
         A single aesara variable or a list of aesara variables that the
         function should return as the output if ``condition`` evaluates to
         true. The number of variables should match those in the
         ``else_branch``, and there should be a one to one correspondence
         (type wise) with the tensors provided in the else branch
 
-    :type else_branch: list of aesara expressions/ aesara expressions
-    :param else_branch:
+    else_branch
         A single aesara variable or a list of aesara variables that the
         function should return as the output if ``condition`` evaluates to
         false. The number of variables should match those in the then branch,
-        and there should be a one to one correspondace (type wise) with the
+        and there should be a one to one correspondence (type wise) with the
         tensors provided in the then branch.
 
-    :return:
+    Returns
+    =======
         A list of aesara variables or a single variable (depending on the
         nature of the ``then_branch`` and ``else_branch``). More exactly if
         ``then_branch`` and ``else_branch`` is a tensor, then
@@ -359,38 +364,13 @@ def ifelse(condition, then_branch, else_branch, name=None):
     new_else_branch = []
     for then_branch_elem, else_branch_elem in zip(then_branch, else_branch):
         if not isinstance(then_branch_elem, Variable):
-            then_branch_elem = aet.basic.as_tensor_variable(then_branch_elem)
+            then_branch_elem = at.basic.as_tensor_variable(then_branch_elem)
         if not isinstance(else_branch_elem, Variable):
-            else_branch_elem = aet.basic.as_tensor_variable(else_branch_elem)
+            else_branch_elem = at.basic.as_tensor_variable(else_branch_elem)
 
-        if then_branch_elem.type != else_branch_elem.type:
-            # If one of them is a TensorType, and the other one can be
-            # converted into one, then we try to do that.
-            # This case happens when one of the elements has a GPU type,
-            # for instance a shared variable that was silently moved to GPU.
-            if isinstance(then_branch_elem.type, TensorType) and not isinstance(
-                else_branch_elem.type, TensorType
-            ):
-                else_branch_elem = then_branch_elem.type.filter_variable(
-                    else_branch_elem
-                )
-
-            elif isinstance(else_branch_elem.type, TensorType) and not isinstance(
-                then_branch_elem.type, TensorType
-            ):
-                then_branch_elem = else_branch_elem.type.filter_variable(
-                    then_branch_elem
-                )
-
-            if then_branch_elem.type != else_branch_elem.type:
-                # If the types still don't match, there is a problem.
-                raise TypeError(
-                    "The two branches should have identical types, but "
-                    f"they are {then_branch_elem.type} and {else_branch_elem.type} respectively. This error could be "
-                    "raised if for example you provided a one element "
-                    "list on the `then` branch but a tensor on the `else` "
-                    "branch."
-                )
+        # Make sure the types are compatible
+        else_branch_elem = then_branch_elem.type.filter_variable(else_branch_elem)
+        then_branch_elem = else_branch_elem.type.filter_variable(then_branch_elem)
 
         new_then_branch.append(then_branch_elem)
         new_else_branch.append(else_branch_elem)
@@ -428,7 +408,7 @@ def cond_make_inplace(fgraph, node):
         # optimization.
         (
             len(fgraph.apply_nodes) < 500
-            or not all([getattr(o.type, "ndim", -1) == 0 for o in node.outputs])
+            or not all(getattr(o.type, "ndim", -1) == 0 for o in node.outputs)
         )
     ):
         return IfElse(n_outs=op.n_outs, as_view=True, gpu=op.gpu, name=op.name)(
@@ -485,13 +465,13 @@ acceptable_ops = (
     SpecifyShape,
     Reshape,
     basic.Rebroadcast,
-    aet.math.Dot,
-    aet.math.MaxAndArgmax,
-    aet.subtensor.Subtensor,
-    aet.subtensor.IncSubtensor,
-    aet.basic.Alloc,
-    aet.elemwise.Elemwise,
-    aet.elemwise.DimShuffle,
+    at.math.Dot,
+    at.math.MaxAndArgmax,
+    at.subtensor.Subtensor,
+    at.subtensor.IncSubtensor,
+    at.basic.Alloc,
+    at.elemwise.Elemwise,
+    at.elemwise.DimShuffle,
 )
 
 
@@ -637,11 +617,11 @@ class CondMerge(GlobalOptimizer):
                 new_outs = new_ifelse(*new_ins, return_list=True)
                 new_outs = [clone_replace(x) for x in new_outs]
                 old_outs = []
-                if type(merging_node.outputs) not in (list, tuple):
+                if not isinstance(merging_node.outputs, (list, tuple)):
                     old_outs += [merging_node.outputs]
                 else:
                     old_outs += merging_node.outputs
-                if type(proposal.outputs) not in (list, tuple):
+                if not isinstance(proposal.outputs, (list, tuple)):
                     old_outs += [proposal.outputs]
                 else:
                     old_outs += proposal.outputs
@@ -737,11 +717,11 @@ def cond_merge_random_op(fgraph, main_node):
             )
             new_outs = new_ifelse(*new_ins, return_list=True)
             old_outs = []
-            if type(merging_node.outputs) not in (list, tuple):
+            if not isinstance(merging_node.outputs, (list, tuple)):
                 old_outs += [merging_node.outputs]
             else:
                 old_outs += merging_node.outputs
-            if type(proposal.outputs) not in (list, tuple):
+            if not isinstance(proposal.outputs, (list, tuple)):
                 old_outs += [proposal.outputs]
             else:
                 old_outs += proposal.outputs
