@@ -413,7 +413,7 @@ def jax_funcify_Composite(op, vectorize=True, **kwargs):
 @jax_funcify.register(Scan)
 def jax_funcify_Scan(op, **kwargs):
     inner_fg = FunctionGraph(op.inputs, op.outputs)
-    jax_aet_inner_func = jax_funcify(inner_fg, **kwargs)
+    jax_at_inner_func = jax_funcify(inner_fg, **kwargs)
 
     def scan(*outer_inputs):
         scan_args = ScanArgs(
@@ -532,7 +532,7 @@ def jax_funcify_Scan(op, **kwargs):
 
         def jax_inner_func(carry, x):
             inner_args = jax_args_to_inner_scan(op, carry, x)
-            inner_scan_outs = list(jax_aet_inner_func(*inner_args))
+            inner_scan_outs = list(jax_at_inner_func(*inner_args))
             new_carry = inner_scan_outs_to_jax_outs(op, carry, inner_scan_outs)
             return new_carry, inner_scan_outs
 
