@@ -1,4 +1,5 @@
 import inspect
+from functools import singledispatch
 from numbers import Number
 from textwrap import indent
 from typing import Union
@@ -7,22 +8,123 @@ import numba
 import numpy as np
 from numba.cpython.unsafe.tuple import tuple_setitem
 
+from aesara import config
 from aesara.graph.basic import Apply
+from aesara.graph.op import Op
 from aesara.link.numba.dispatch import basic as numba_basic
 from aesara.link.numba.dispatch.basic import (
     create_numba_signature,
     create_tuple_creator,
     numba_funcify,
+    use_optimized_cheap_pass,
 )
 from aesara.link.utils import (
     compile_function_src,
     get_name_for_object,
     unique_name_generator,
 )
+from aesara.scalar.basic import (
+    AND,
+    OR,
+    XOR,
+    Add,
+    IntDiv,
+    Mean,
+    Mul,
+    ScalarMaximum,
+    ScalarMinimum,
+    Sub,
+    TrueDiv,
+)
+from aesara.scalar.basic import add as add_as
+from aesara.scalar.basic import scalar_maximum
 from aesara.tensor.elemwise import CAReduce, DimShuffle, Elemwise
-from aesara.tensor.math import MaxAndArgmax
+from aesara.tensor.math import MaxAndArgmax, MulWithoutZeros
 from aesara.tensor.nnet.basic import LogSoftmax, Softmax, SoftmaxGrad
-from aesara.tensor.type import tensor
+
+
+@singledispatch
+def scalar_in_place_fn(op: Op, idx: str, res: str, arr: str):
+    """Return code for an in-place update on an array using a binary scalar :class:`Op`.
+
+    Parameters
+    ----------
+    op
+        The scalar :class:`Op`
+    idx
+        The index of `res` that needs to be updated.
+    res
+        The symbol name for the first input and results/output.
+    arr
+        The symbol name for the second input.
+    """
+    raise NotImplementedError()
+
+
+@scalar_in_place_fn.register(Add)
+def scalar_in_place_fn_Add(op, idx, res, arr):
+    return f"{res}[{idx}] += {arr}"
+
+
+@scalar_in_place_fn.register(Sub)
+def scalar_in_place_fn_Sub(op, idx, res, arr):
+    return f"{res}[{idx}] -= {arr}"
+
+
+@scalar_in_place_fn.register(Mean)
+def scalar_in_place_fn_Mean(op, idx, res, arr):
+    return f"{res}[{idx}] += ({arr} - {res}[{idx}]) / (i + 1)"
+
+
+@scalar_in_place_fn.register(Mul)
+def scalar_in_place_fn_Mul(op, idx, res, arr):
+    return f"{res}[{idx}] *= {arr}"
+
+
+@scalar_in_place_fn.register(MulWithoutZeros)
+def scalar_in_place_fn_MulWithoutZeros(op, idx, res, arr):
+    return f"{res}[{idx}] = {arr} if {res}[{idx}] == 0 else ({res}[{idx}] if {arr} == 0 else {res}[{idx}] * {arr})"
+
+
+@scalar_in_place_fn.register(AND)
+def scalar_in_place_fn_AND(op, idx, res, arr):
+    return f"{res}[{idx}] &= {arr}"
+
+
+@scalar_in_place_fn.register(OR)
+def scalar_in_place_fn_OR(op, idx, res, arr):
+    return f"{res}[{idx}] |= {arr}"
+
+
+@scalar_in_place_fn.register(XOR)
+def scalar_in_place_fn_XOR(op, idx, res, arr):
+    return f"{res}[{idx}] ^= {arr}"
+
+
+@scalar_in_place_fn.register(TrueDiv)
+def scalar_in_place_fn_TrueDiv(op, idx, res, arr):
+    return f"{res}[{idx}] /= {arr}"
+
+
+@scalar_in_place_fn.register(IntDiv)
+def scalar_in_place_fn_IntDiv(op, idx, res, arr):
+    return f"{res}[{idx}] //= {arr}"
+
+
+@scalar_in_place_fn.register(ScalarMaximum)
+def scalar_in_place_fn_ScalarMaximum(op, idx, res, arr):
+    return f"""
+if {res}[{idx}] < {arr}:
+    {res}[{idx}] = {arr}
+"""
+
+
+@scalar_in_place_fn.register(ScalarMinimum)
+def scalar_in_place_fn_ScalarMinimum(op, idx, res, arr):
+    return f"""
+if {res}[{idx}] > {arr}:
+    {res}[{idx}] = {arr}
+"""
 
 
 def create_vectorize_func(op, node, use_signature=False, identity=None, **kwargs):
@@ -38,13 +140,283 @@ def create_vectorize_func(op, node, use_signature=False, identity=None, **kwargs
     else:
         signature = []
 
-    numba_vectorize = numba.vectorize(signature, identity=identity)
-    elemwise_fn = numba_vectorize(scalar_op_fn)
-    elemwise_fn.py_scalar_func = scalar_op_fn
+    target = (
+        getattr(node.tag, "numba__vectorize_target", None)
+        or config.numba__vectorize_target
+    )
+
+    numba_vectorized_fn = numba_basic.numba_vectorize(
+        signature, identity=identity, target=target, fastmath=config.numba__fastmath
+    )
+
+    py_scalar_func = getattr(scalar_op_fn, "py_func", scalar_op_fn)
+
+    elemwise_fn = numba_vectorized_fn(scalar_op_fn)
+    elemwise_fn.py_scalar_func = py_scalar_func
 
     return elemwise_fn
 
 
+def create_axis_reducer(
+    scalar_op: Op,
+    identity: Union[np.ndarray, Number],
+    axis: int,
+    ndim: int,
+    dtype: numba.types.Type,
+    keepdims: bool = False,
+) -> numba.core.dispatcher.Dispatcher:
+    r"""Create Python function that performs a NumPy-like reduction on a given axis.
+
+    The functions generated by this function take the following form:
+
+    .. code-block:: python
+
+        def careduce_axis(x):
+            res_shape = tuple(shape[i] if i < axis else shape[i + 1] for i in range(ndim - 1))
+            res = np.full(res_shape, identity, dtype=dtype)
+
+            x_axis_first = x.transpose(reaxis_first)
+
+            for m in range(x.shape[axis]):
+                reduce_fn(res, x_axis_first[m], res)
+
+            if keepdims:
+                return np.expand_dims(res, axis)
+            else:
+                return res
+
+
+    This can be removed/replaced when
+    https://github.com/numba/numba/issues/4504 is implemented.
+
+    Parameters
+    ==========
+    scalar_op:
+        The scalar :class:`Op` that performs the desired reduction.
+    identity:
+        The identity value for the reduction.
+    axis:
+        The axis to reduce.
+    ndim:
+        The number of dimensions of the result.
+    dtype:
+        The data type of the result.
+    keepdims:
+        Determines whether or not the reduced dimension is retained.
+
+
+    Returns
+    =======
+    A Python function that can be JITed.
+
+    """
+
+    reduce_elemwise_fn_name = "careduce_axis"
+
+    identity = str(identity)
+    if identity == "inf":
+        identity = "np.inf"
+    elif identity == "-inf":
+        identity = "-np.inf"
+
+    global_env = {
+        "np": np,
+        "numba_basic": numba_basic,
+        "out_dtype": dtype,
+    }
+
+    if ndim > 1:
+        res_shape_tuple_ctor = create_tuple_creator(
+            lambda i, shape: shape[i] if i < axis else shape[i + 1], ndim - 1
+        )
+        global_env["res_shape_tuple_ctor"] = res_shape_tuple_ctor
+
+        res_indices = []
+        arr_indices = []
+        count = 0
+
+        for i in range(ndim):
+            if i == axis:
+                arr_indices.append("i")
+            else:
+                res_indices.append(f"idx_arr[{count}]")
+                arr_indices.append(f"idx_arr[{count}]")
+                count = count + 1
+
+        res_indices = ", ".join(res_indices)
+        arr_indices = ", ".join(arr_indices)
+
+        inplace_update_statement = scalar_in_place_fn(
+            scalar_op, res_indices, "res", f"x[{arr_indices}]"
+        )
+        inplace_update_statement = indent(inplace_update_statement, " " * 4 * 3)
+
+        return_expr = f"np.expand_dims(res, {axis})" if keepdims else "res"
+        reduce_elemwise_def_src = f"""
+def {reduce_elemwise_fn_name}(x):
+
+    x_shape = np.shape(x)
+    res_shape = res_shape_tuple_ctor(x_shape)
+    res = np.full(res_shape, numba_basic.to_scalar({identity}), dtype=out_dtype)
+
+    axis_shape = x.shape[{axis}]
+
+    for idx_arr in np.ndindex(res_shape):
+        for i in range(axis_shape):
+{inplace_update_statement}
+
+    return {return_expr}
+        """
+    else:
+        inplace_update_statement = scalar_in_place_fn(scalar_op, "0", "res", "x[i]")
+        inplace_update_statement = indent(inplace_update_statement, " " * 4 * 2)
+
+        return_expr = "res" if keepdims else "res.item()"
+        reduce_elemwise_def_src = f"""
+def {reduce_elemwise_fn_name}(x):
+
+    res = np.full(1, numba_basic.to_scalar({identity}), dtype=out_dtype)
+
+    axis_shape = x.shape[{axis}]
+
+    for i in range(axis_shape):
+{inplace_update_statement}
+
+    return {return_expr}
+        """
+
+    reduce_elemwise_fn_py = compile_function_src(
+        reduce_elemwise_def_src, reduce_elemwise_fn_name, {**globals(), **global_env}
+    )
+
+    return reduce_elemwise_fn_py
+
+
+def create_multiaxis_reducer(
+    scalar_op, identity, axes, ndim, dtype, input_name="input"
+):
+    r"""Construct a function that reduces multiple axes.
+
+    The functions generated by this function take the following form:
+
+    .. code-block:: python
+
+        def careduce_maximum(input):
+            axis_0_res = careduce_axes_fn_0(input)
+            axis_1_res = careduce_axes_fn_1(axis_0_res)
+            ...
+            axis_N_res = careduce_axes_fn_N(axis_N_minus_1_res)
+            return axis_N_res
+
+    The range 0-N is determined by the `axes` argument (i.e. the
+    axes to be reduced).
+
+
+    Parameters
+    ==========
+    scalar_op:
+        The scalar :class:`Op` that performs the desired reduction.
+    identity:
+        The identity value for the reduction.
+    axes:
+        The axes to reduce.
+    ndim:
+        The number of dimensions of the result.
+    dtype:
+        The data type of the result.
+
+    Returns
+    =======
+    A Python function that can be JITed.
+
+    """
+    if len(axes) == 1:
+        return create_axis_reducer(scalar_op, identity, axes[0], ndim, dtype)
+
+    careduce_fn_name = f"careduce_{scalar_op}"
+    global_env = {}
+    to_reduce = reversed(sorted(axes))
+    careduce_lines_src = []
+    var_name = input_name
+
+    for i, axis in enumerate(to_reduce):
+        careducer_axes_fn_name = f"careduce_axes_fn_{i}"
+        reducer_py_fn = create_axis_reducer(scalar_op, identity, axis, ndim, dtype)
+        reducer_fn = numba_basic.numba_njit(
+            boundscheck=False, fastmath=config.numba__fastmath
+        )(reducer_py_fn)
+
+        global_env[careducer_axes_fn_name] = reducer_fn
+
+        ndim -= 1
+        last_var_name = var_name
+        var_name = f"axis_{i}_res"
+        careduce_lines_src.append(
+            f"{var_name} = {careducer_axes_fn_name}({last_var_name})"
+        )
+
+    careduce_assign_lines = indent("\n".join(careduce_lines_src), " " * 4)
+    careduce_def_src = f"""
+def {careduce_fn_name}({input_name}):
+{careduce_assign_lines}
+    return {var_name}
+    """
+
+    careduce_fn = compile_function_src(
+        careduce_def_src, careduce_fn_name, {**globals(), **global_env}
+    )
+
+    return careduce_fn
+
+
+def jit_compile_reducer(node, fn, **kwds):
+    """Compile Python source for reduction loops using additional optimizations.
+
+    Parameters
+    ==========
+    node
+        An node from which the signature can be derived.
+    fn
+        The Python function object to compile.
+    kwds
+        Extra keywords to be added to the :func:`numba.njit` function.
+
+    Returns
+    =======
+    A :func:`numba.njit`-compiled function.
+
+    """
+    signature = create_numba_signature(node, reduce_to_scalar=True)
+
+    # Eagerly compile the function using increased optimizations.  This should
+    # help improve nested loop reductions.
+    with use_optimized_cheap_pass():
+        res = numba_basic.numba_njit(
+            signature,
+            boundscheck=False,
+            fastmath=config.numba__fastmath,
+            **kwds,
+        )(fn)
+
+    return res
+
+
+def create_axis_apply_fn(fn, axis, ndim, dtype):
+    reaxis_first = tuple(i for i in range(ndim) if i != axis) + (axis,)
+
+    @numba_basic.numba_njit(boundscheck=False)
+    def axis_apply_fn(x):
+        x_reaxis = x.transpose(reaxis_first)
+
+        res = np.zeros(x_reaxis.shape[:-1], dtype=dtype)
+        for m in np.ndindex(res.shape):
+            v = fn(x_reaxis[m])
+            res[m] = v
+        return res
+
+    return axis_apply_fn
+
+
 @numba_funcify.register(Elemwise)
 def numba_funcify_Elemwise(op, node, **kwargs):
 
@@ -85,199 +457,17 @@ def {inplace_elemwise_fn_name}({input_signature_str}):
             """
 
         inplace_elemwise_fn = compile_function_src(
-            inplace_elemwise_src, inplace_elemwise_fn_name, inplace_global_env
+            inplace_elemwise_src,
+            inplace_elemwise_fn_name,
+            {**globals(), **inplace_global_env},
+        )
+        return numba_basic.numba_njit(inline="always", fastmath=config.numba__fastmath)(
+            inplace_elemwise_fn
         )
-        return numba.njit(inline="always")(inplace_elemwise_fn)
 
     return elemwise_fn
 
 
-def create_axis_reducer(
-    reduce_fn: numba.np.ufunc.dufunc.DUFunc,
-    identity: Union[np.ndarray, Number],
-    axis: int,
-    ndim: int,
-    dtype: numba.types.Type,
-    keepdims: bool = False,
-) -> numba.core.dispatcher.Dispatcher:
-    r"""Create a Numba JITed function that performs a NumPy reduction on a given axis.
-
-    The functions generated by this function take the following form:
-
-    .. code-block:: python
-
-        def careduce_axis(x):
-            res_shape = tuple(shape[i] if i < axis else shape[i + 1] for i in range(ndim - 1))
-            res = np.full(res_shape, identity, dtype=dtype)
-
-            x_axis_first = x.transpose(reaxis_first)
-
-            for m in range(x.shape[axis]):
-                reduce_fn(res, x_axis_first[m], res)
-
-            if keepdims:
-                return np.expand_dims(res, axis)
-            else:
-                return res
-
-
-    This can be removed/replaced when
-    https://github.com/numba/numba/issues/4504 is implemented.
-
-    Parameters
-    ==========
-    reduce_fn:
-        The Numba ``ufunc`` representing a binary op that can perform the
-        reduction on arbitrary ``ndarray``\s.
-    identity:
-        The identity value for the reduction.
-    axis:
-        The axis to reduce.
-    ndim:
-        The number of dimensions of the result.
-    dtype:
-        The data type of the result.
-    keepdims:
-        Determines whether or not the reduced dimension is retained.
-    """
-    if ndim > 1:
-
-        if keepdims:
-
-            @numba.njit(inline="always")
-            def set_out_dims(x):
-                return np.expand_dims(x, axis)
-
-        else:
-
-            @numba.njit(inline="always")
-            def set_out_dims(x):
-                return x
-
-        res_shape_tuple_ctor = create_tuple_creator(
-            lambda i, shape: shape[i] if i < axis else shape[i + 1], ndim - 1
-        )
-
-        reaxis_first = (axis,) + tuple(i for i in range(ndim) if i != axis)
-
-        @numba.njit(boundscheck=False)
-        def careduce_axis(x):
-            res_shape = res_shape_tuple_ctor(x.shape)
-            x_axis_first = x.transpose(reaxis_first)
-
-            res = np.full(res_shape, numba_basic.to_scalar(identity), dtype=dtype)
-            for m in range(x.shape[axis]):
-                reduce_fn(res, x_axis_first[m], res)
-
-            return set_out_dims(res)
-
-    else:
-
-        if keepdims:
-
-            @numba.njit(inline="always")
-            def set_out_dims(x):
-                return np.array([x], dtype)
-
-        else:
-
-            @numba.njit(inline="always")
-            def set_out_dims(x):
-                return numba_basic.direct_cast(x, dtype)
-
-        @numba.njit(boundscheck=False)
-        def careduce_axis(x):
-            res = numba_basic.to_scalar(identity)
-            for val in x:
-                res = reduce_fn(res, val)
-            return set_out_dims(res)
-
-    return careduce_axis
-
-
-def create_multiaxis_reducer(
-    reduce_fn, identity, axes, ndim, dtype, input_name="input"
-):
-    r"""Construct a function that reduces multiple axes.
-
-    The functions generated by this function take the following form:
-
-    .. code-block:: python
-
-        def careduce_maximum(input):
-            axis_0_res = careduce_axes_fn_0(input)
-            axis_1_res = careduce_axes_fn_1(axis_0_res)
-            ...
-            axis_N_res = careduce_axes_fn_N(axis_N_minus_1_res)
-            return axis_N_res
-
-    The range 0-N is determined by the `axes` argument (i.e. the
-    axes to be reduced).
-
-
-    Parameters
-    ==========
-    reduce_fn:
-        The Numba ``ufunc`` representing a binary op that can perform the
-        reduction on arbitrary ``ndarray``\s.
-    identity:
-        The identity value for the reduction.
-    axes:
-        The axes to reduce.
-    ndim:
-        The number of dimensions of the result.
-    dtype:
-        The data type of the result.
-
-    """
-    if len(axes) == 1:
-        return create_axis_reducer(reduce_fn, identity, axes[0], ndim, dtype)
-
-    careduce_fn_name = f"careduce_{get_name_for_object(reduce_fn)}"
-    global_env = {}
-    to_reduce = reversed(sorted(axes))
-    careduce_lines_src = []
-    var_name = input_name
-
-    for i, axis in enumerate(to_reduce):
-        careducer_axes_fn_name = f"careduce_axes_fn_{i}"
-        global_env[careducer_axes_fn_name] = create_axis_reducer(
-            reduce_fn, identity, axis - i, ndim, dtype
-        )
-        ndim -= 1
-        last_var_name = var_name
-        var_name = f"axis_{i}_res"
-        careduce_lines_src.append(
-            f"{var_name} = {careducer_axes_fn_name}({last_var_name})"
-        )
-
-    careduce_assign_lines = indent("\n".join(careduce_lines_src), " " * 4)
-    careduce_def_src = f"""
-def {careduce_fn_name}({input_name}):
-{careduce_assign_lines}
-    return {var_name}
-    """
-
-    careduce_fn = compile_function_src(careduce_def_src, careduce_fn_name, global_env)
-    return numba.njit(careduce_fn)
-
-
-def create_axis_apply_fn(fn, axis, ndim, dtype):
-    reaxis_first = tuple(i for i in range(ndim) if i != axis) + (axis,)
-
-    @numba.njit(boundscheck=False)
-    def axis_apply_fn(x):
-        x_reaxis = x.transpose(reaxis_first)
-
-        res = np.zeros(x_reaxis.shape[:-1], dtype=dtype)
-        for m in np.ndindex(res.shape):
-            v = fn(x_reaxis[m])
-            res[m] = v
-        return res
-
-    return axis_apply_fn
-
-
 @numba_funcify.register(CAReduce)
 def numba_funcify_CAReduce(op, node, **kwargs):
     axes = op.axis
@@ -293,26 +483,18 @@ def numba_funcify_CAReduce(op, node, **kwargs):
 
     scalar_op_identity = np.asarray(op.scalar_op.identity, dtype=np_acc_dtype)
 
-    scalar_nfunc_spec = op.scalar_op.nfunc_spec
-
-    # We construct a dummy `Apply` that has the minimum required number of
-    # inputs for the scalar `Op`.  Without this, we would get a scalar function
-    # with too few arguments.
-    dummy_node = Apply(
-        op,
-        [tensor(np_acc_dtype, [False]) for i in range(scalar_nfunc_spec[1])],
-        [tensor(np_acc_dtype, [False]) for o in range(scalar_nfunc_spec[2])],
-    )
-
-    # TODO: Use `scalar_op_identity`?
-    elemwise_fn = create_vectorize_func(op, dummy_node, use_signature=True, **kwargs)
-
     input_name = get_name_for_object(node.inputs[0])
     ndim = node.inputs[0].ndim
-    careduce_fn = create_multiaxis_reducer(
-        elemwise_fn, scalar_op_identity, axes, ndim, np_acc_dtype, input_name=input_name
+    careduce_py_fn = create_multiaxis_reducer(
+        op.scalar_op,
+        scalar_op_identity,
+        axes,
+        ndim,
+        np.dtype(node.outputs[0].type.dtype),
+        input_name=input_name,
     )
 
+    careduce_fn = jit_compile_reducer(node, careduce_py_fn)
     return careduce_fn
 
 
@@ -327,7 +509,7 @@ def numba_funcify_DimShuffle(op, **kwargs):
 
     if len(shuffle) > 0:
 
-        @numba.njit
+        @numba_basic.numba_njit
         def populate_new_shape(i, j, new_shape, shuffle_shape):
             if i in augment:
                 new_shape = tuple_setitem(new_shape, i, 1)
@@ -341,7 +523,7 @@ def numba_funcify_DimShuffle(op, **kwargs):
         # is typed as `getitem(Tuple(), int)`, which has no implementation
         # (since getting an item from an empty sequence doesn't make sense).
         # To avoid this compile-time error, we omit the expression altogether.
-        @numba.njit(inline="always")
+        @numba_basic.numba_njit(inline="always")
         def populate_new_shape(i, j, new_shape, shuffle_shape):
             return j, tuple_setitem(new_shape, i, 1)
 
@@ -350,7 +532,7 @@ def numba_funcify_DimShuffle(op, **kwargs):
             lambda _: 0, ndim_new_shape
         )
 
-        @numba.njit
+        @numba_basic.numba_njit
         def dimshuffle_inner(x, shuffle):
             res = np.transpose(x, transposition)
             shuffle_shape = res.shape[: len(shuffle)]
@@ -371,7 +553,7 @@ def numba_funcify_DimShuffle(op, **kwargs):
 
     else:
 
-        @numba.njit
+        @numba_basic.numba_njit
         def dimshuffle_inner(x, shuffle):
             return x.item()
 
@@ -387,7 +569,7 @@ def numba_funcify_DimShuffle(op, **kwargs):
     # E       No match.
     # ...(on this line)...
     # E           shuffle_shape = res.shape[: len(shuffle)]
-    @numba.njit(inline="always")
+    @numba_basic.numba_njit(inline="always")
     def dimshuffle(x):
         return dimshuffle_inner(np.asarray(x), shuffle)
 
@@ -403,24 +585,31 @@ def numba_funcify_Softmax(op, node, **kwargs):
     axis = op.axis
 
     if axis is not None:
-        reduce_max = create_axis_reducer(
-            np.maximum, -np.inf, axis, x_at.ndim, x_dtype, keepdims=True
+        reduce_max_py = create_axis_reducer(
+            scalar_maximum, -np.inf, axis, x_at.ndim, x_dtype, keepdims=True
         )
-        reduce_sum = create_axis_reducer(
-            np.add, 0.0, axis, x_at.ndim, x_dtype, keepdims=True
+        reduce_sum_py = create_axis_reducer(
+            add_as, 0.0, axis, x_at.ndim, x_dtype, keepdims=True
         )
+
+        jit_fn = numba_basic.numba_njit(
+            boundscheck=False, fastmath=config.numba__fastmath
+        )
+        reduce_max = jit_fn(reduce_max_py)
+        reduce_sum = jit_fn(reduce_sum_py)
     else:
         reduce_max = np.max
         reduce_sum = np.sum
 
-    @numba.njit
-    def softmax(x):
+    def softmax_py_fn(x):
         z = reduce_max(x)
         e_x = np.exp(x - z)
         w = reduce_sum(e_x)
         sm = e_x / w
         return sm
 
+    softmax = jit_compile_reducer(node, softmax_py_fn)
+
     return softmax
 
 
@@ -433,19 +622,25 @@ def numba_funcify_SoftmaxGrad(op, node, **kwargs):
 
     axis = op.axis
     if axis is not None:
-        reduce_sum = create_axis_reducer(
-            np.add, 0.0, axis, sm_at.ndim, sm_dtype, keepdims=True
+        reduce_sum_py = create_axis_reducer(
+            add_as, 0.0, axis, sm_at.ndim, sm_dtype, keepdims=True
         )
+
+        jit_fn = numba_basic.numba_njit(
+            boundscheck=False, fastmath=config.numba__fastmath
+        )
+        reduce_sum = jit_fn(reduce_sum_py)
     else:
         reduce_sum = np.sum
 
-    @numba.njit
-    def softmax_grad(dy, sm):
+    def softmax_grad_py_fn(dy, sm):
         dy_times_sm = dy * sm
         sum_dy_times_sm = reduce_sum(dy_times_sm)
         dx = dy_times_sm - sum_dy_times_sm * sm
         return dx
 
+    softmax_grad = jit_compile_reducer(node, softmax_grad_py_fn)
+
     return softmax_grad
 
 
@@ -458,22 +653,28 @@ def numba_funcify_LogSoftmax(op, node, **kwargs):
     axis = op.axis
 
     if axis is not None:
-        reduce_max = create_axis_reducer(
-            np.maximum, -np.inf, axis, x_at.ndim, x_dtype, keepdims=True
+        reduce_max_py = create_axis_reducer(
+            scalar_maximum, -np.inf, axis, x_at.ndim, x_dtype, keepdims=True
         )
-        reduce_sum = create_axis_reducer(
-            np.add, 0.0, axis, x_at.ndim, x_dtype, keepdims=True
+        reduce_sum_py = create_axis_reducer(
+            add_as, 0.0, axis, x_at.ndim, x_dtype, keepdims=True
         )
+
+        jit_fn = numba_basic.numba_njit(
+            boundscheck=False, fastmath=config.numba__fastmath
+        )
+        reduce_max = jit_fn(reduce_max_py)
+        reduce_sum = jit_fn(reduce_sum_py)
     else:
         reduce_max = np.max
         reduce_sum = np.sum
 
-    @numba.njit
-    def log_softmax(x):
+    def log_softmax_py_fn(x):
         xdev = x - reduce_max(x)
         lsm = xdev - np.log(reduce_sum(np.exp(xdev)))
         return lsm
 
+    log_softmax = jit_compile_reducer(node, log_softmax_py_fn)
     return log_softmax
 
 
@@ -487,7 +688,7 @@ def numba_funcify_MaxAndArgmax(op, node, **kwargs):
 
     if x_ndim == 0:
 
-        @numba.njit(inline="always")
+        @numba_basic.numba_njit(inline="always")
         def maxandargmax(x):
             return x, 0
 
@@ -499,9 +700,13 @@ def numba_funcify_MaxAndArgmax(op, node, **kwargs):
         # work-around
         keep_axes = tuple(i for i in range(x_ndim) if i not in axes)
 
-        reduce_max = create_multiaxis_reducer(
-            np.maximum, -np.inf, axes, x_ndim, x_dtype
+        reduce_max_py_fn = create_multiaxis_reducer(
+            scalar_maximum, -np.inf, axes, x_ndim, x_dtype
         )
+        reduce_max = jit_compile_reducer(
+            Apply(node.op, node.inputs, [node.outputs[0].clone()]), reduce_max_py_fn
+        )
+
         reduced_x_ndim = x_ndim - len(axes) + 1
         argmax_axis = create_axis_apply_fn(
             np.argmax, reduced_x_ndim - 1, reduced_x_ndim, np.int64
@@ -511,7 +716,7 @@ def numba_funcify_MaxAndArgmax(op, node, **kwargs):
         sl1 = slice(None, len(keep_axes))
         sl2 = slice(len(keep_axes), None)
 
-        @numba.njit
+        @numba_basic.numba_njit
         def maxandargmax(x):
             max_res = reduce_max(x)
 
