@@ -14,8 +14,7 @@ from typing import Any, Callable, Dict, Iterable, List, NoReturn, Optional, Tupl
 
 import numpy as np
 
-from aesara import utils
-from aesara.configdefaults import config
+from aesara import config, utils
 from aesara.graph.basic import Apply, Constant, Variable
 from aesara.graph.fg import FunctionGraph
 
@@ -254,7 +253,7 @@ def gc_helper(node_list: List[Apply]):
 
 
 def raise_with_op(
-    fgraph: FunctionGraph, node, thunk=None, exc_info=None, storage_map=None
+    fgraph: FunctionGraph, node: Apply, thunk=None, exc_info=None, storage_map=None
 ):
     """
     Re-raise an exception while annotating the exception object with
@@ -262,12 +261,16 @@ def raise_with_op(
 
     Parameters
     ----------
+    fgraph
+        The `FunctionGraph` that contains `node`.
     node : Apply node
         The Apply node object that resulted in the raised exception.
+    thunk :
+        An optional thunk.
     exc_info : tuple, optional
         A tuple containing the exception type, exception object and
         associated traceback, as would be returned by a call to
-        `sys.exc_info()` (which is done if `None` is passed).
+        `sys.exc_info` (which is done if ``None`` is passed).
     storage_map: dict, optional
         storage map of the aesara function that resulted in the
         raised exception.
@@ -279,12 +282,12 @@ def raise_with_op(
     object with several new members which may be helpful for debugging
     Aesara graphs. They are:
 
-     * __op_instance__: The Op that is responsible for the exception
+     * ``__op_instance__``: The `Op` that is responsible for the exception
        being raised.
-     * __thunk_trace__: A traceback corresponding to the code that
+     * ``__thunk_trace__``: A traceback corresponding to the code that
        actually generated the exception, if it is available.
-     * __applynode_index__: The index of the Apply node corresponding
-       to this op in `op.fgraph.toposort()`.
+     * ``__applynode_index__``: The index of the `Apply` node corresponding
+       to this `Op` in ``op.fgraph.toposort``.
 
     The exception is not annotated if it is of type `KeyboardInterrupt`.
 
@@ -588,7 +591,9 @@ def compile_function_src(src, function_name, global_env=None, local_env=None):
     mod_code = compile(src, filename, mode="exec")
     exec(mod_code, global_env, local_env)
 
-    return local_env[function_name]
+    res = local_env[function_name]
+    res.__source__ = src
+    return res
 
 
 def get_name_for_object(x: Any):
@@ -762,7 +767,7 @@ def {fgraph_name}({", ".join(fgraph_input_names)}):
         local_env = locals()
 
     fgraph_def = compile_function_src(
-        fgraph_def_src, fgraph_name, global_env, local_env
+        fgraph_def_src, fgraph_name, {**globals(), **global_env}, local_env
     )
 
     return fgraph_def
