@@ -3,10 +3,9 @@ from collections import OrderedDict
 
 import numpy as np
 
-import aesara.tensor as aet
+import aesara.tensor as at
 from aesara.compile import SharedVariable
-from aesara.compile.function import function
-from aesara.compile.mode import Mode
+from aesara.compile.function.pfunc import construct_pfunc_ins_and_outs
 from aesara.configdefaults import config
 from aesara.graph.basic import Constant, Variable, clone_replace, graph_inputs
 from aesara.graph.fg import MissingInputError
@@ -103,13 +102,13 @@ def scan(
 
         .. code-block:: python
 
-            import aesara.tensor as aet
+            import aesara.tensor as at
 
-            W   = aet.matrix()
+            W   = at.matrix()
             W_2 = W**2
 
             def f(x):
-                return aet.dot(x,W_2)
+                return at.dot(x,W_2)
 
         The function `fn` is expected to return two things. One is a list of
         outputs ordered in the same order as `outputs_info`, with the
@@ -331,7 +330,7 @@ def scan(
     non_seqs = []
     for elem in wrap_into_list(non_sequences):
         if not isinstance(elem, Variable):
-            non_seqs.append(aet.as_tensor_variable(elem))
+            non_seqs.append(at.as_tensor_variable(elem))
         else:
             non_seqs.append(elem)
 
@@ -345,7 +344,7 @@ def scan(
         n_fixed_steps = int(n_steps)
     else:
         try:
-            n_fixed_steps = aet.get_scalar_constant_value(n_steps)
+            n_fixed_steps = at.get_scalar_constant_value(n_steps)
         except NotScalarConstantError:
             n_fixed_steps = None
 
@@ -466,7 +465,7 @@ def scan(
                 # If not we need to use copies, that will be replaced at
                 # each frame by the corresponding slice
                 actual_slice = seq["input"][k - mintap_proxy]
-                _seq_val = aet.as_tensor_variable(seq["input"])
+                _seq_val = at.as_tensor_variable(seq["input"])
                 _seq_val_slice = _seq_val[k - mintap_proxy]
                 nw_slice = _seq_val_slice.type()
 
@@ -528,7 +527,7 @@ def scan(
 
     if not utils.isNaN_or_Inf_or_None(n_steps):
         # ^ N_steps should also be considered
-        lengths_vec.append(aet.as_tensor(n_steps))
+        lengths_vec.append(at.as_tensor(n_steps))
 
     if len(lengths_vec) == 0:
         # ^ No information about the number of steps
@@ -546,7 +545,7 @@ def scan(
         for contestant in lengths_vec[1:]:
             actual_n_steps = minimum(actual_n_steps, contestant)
     else:
-        actual_n_steps = aet.as_tensor(n_steps)
+        actual_n_steps = at.as_tensor(n_steps)
 
     scan_seqs = [seq[:actual_n_steps] for seq in scan_seqs]
     # Conventions :
@@ -594,7 +593,7 @@ def scan(
 
             actual_arg = init_out["initial"]
             if not isinstance(actual_arg, Variable):
-                actual_arg = aet.as_tensor_variable(actual_arg)
+                actual_arg = at.as_tensor_variable(actual_arg)
             arg = safe_new(actual_arg)
             if isinstance(arg, Constant):
                 # safe new returns a clone of the constants, but that is not
@@ -622,7 +621,7 @@ def scan(
             # defined in scan utils
             sit_sot_scan_inputs.append(
                 utils.expand_empty(
-                    aet.unbroadcast(shape_padleft(actual_arg), 0),
+                    at.unbroadcast(shape_padleft(actual_arg), 0),
                     actual_n_steps,
                 )
             )
@@ -655,7 +654,7 @@ def scan(
             for k in init_out["taps"]:
                 # create a new slice
                 actual_nw_slice = init_out["initial"][k + mintap]
-                _init_out_var = aet.as_tensor_variable(init_out["initial"])
+                _init_out_var = at.as_tensor_variable(init_out["initial"])
                 _init_out_var_slice = _init_out_var[k + mintap]
                 nw_slice = _init_out_var_slice.type()
 
@@ -695,7 +694,7 @@ def scan(
     offset = 0
     for idx in range(n_mit_sot):
         n_inputs = len(mit_sot_tap_array[idx])
-        if n_fixed_steps in [1, -1]:
+        if n_fixed_steps in (1, -1):
             _ordered_args[mit_sot_rightOrder[idx]] = mit_sot_inner_slices[
                 offset : offset + n_inputs
             ]
@@ -706,7 +705,7 @@ def scan(
         offset += n_inputs
 
     for idx in range(n_sit_sot):
-        if n_fixed_steps in [1, -1]:
+        if n_fixed_steps in (1, -1):
             _ordered_args[sit_sot_rightOrder[idx]] = [sit_sot_inner_slices[idx]]
         else:
             _ordered_args[sit_sot_rightOrder[idx]] = [sit_sot_inner_inputs[idx]]
@@ -714,7 +713,7 @@ def scan(
     ordered_args = []
     for ls in _ordered_args:
         ordered_args += ls
-    if n_fixed_steps in [1, -1]:
+    if n_fixed_steps in (1, -1):
         args = inner_slices + ordered_args + non_seqs
 
     else:
@@ -740,17 +739,7 @@ def scan(
     # Step 3. Check if we actually need scan and remove it if we don't
     ##
 
-    if n_fixed_steps in [1, -1]:
-        # We do not need to use the scan op anymore, so we can just return
-        # the outputs and updates we have
-        if condition is not None:
-            _logger.warning(
-                (
-                    "When the number of steps is fixed and equal "
-                    f"to 1, the provided stopping condition, {condition} is ignored"
-                )
-            )
-
+    if n_fixed_steps in (1, -1):
         for pos, inner_out in enumerate(outputs):
             # we need to see if we need to pad our sequences with an
             # unbroadcastable dimension; case example : we return an
@@ -759,9 +748,9 @@ def scan(
             # this will represent only a slice and it will have one
             # dimension less.
             if isinstance(inner_out.type, TensorType) and return_steps.get(pos, 0) != 1:
-                outputs[pos] = aet.unbroadcast(shape_padleft(inner_out), 0)
+                outputs[pos] = at.unbroadcast(shape_padleft(inner_out), 0)
 
-        if return_list is not True and len(outputs) == 1:
+        if not return_list and len(outputs) == 1:
             outputs = outputs[0]
 
         return (outputs, updates)
@@ -774,9 +763,6 @@ def scan(
     # we have and what are their update rules (note that the user has
     # the option not to pass the shared variable to scan, so we need to
     # pick them manually and add them to scan)
-    # make the compilation as fast as possible by not applying any
-    # optimization or conversion to C [ note this region is not important
-    # for performance so we can do stuff as unoptimal as we wish ]
 
     # extract still missing inputs (there still might be so) and add them
     # as non sequences at the end of our args
@@ -804,13 +790,8 @@ def scan(
     # Perform a try-except to provide a meaningful error message to the
     # user if inputs of the inner function are missing.
     try:
-        dummy_f = function(
-            dummy_args,
-            dummy_outs,
-            updates=updates,
-            mode=Mode(linker="py", optimizer=None),
-            on_unused_input="ignore",
-            profile=False,
+        dummy_inputs, dummy_outputs = construct_pfunc_ins_and_outs(
+            dummy_args, dummy_outs, updates=updates
         )
     except MissingInputError as err:
         msg = (
@@ -830,7 +811,7 @@ def scan(
     # assumed outputs until now (provided by the user) there can be
     # only one explanation: No information is provided for any of the
     # outputs (i.e. we are dealing with a map)
-    tmp_dummy_f_outs = len(dummy_f.maker.outputs)
+    tmp_dummy_f_outs = len(dummy_outputs)
     if as_while:
         tmp_dummy_f_outs -= 1
     if not (tmp_dummy_f_outs == n_outs or outs_info == []):
@@ -841,7 +822,7 @@ def scan(
         )
 
     if outs_info == []:
-        n_outs = len(dummy_f.maker.outputs)
+        n_outs = len(dummy_outputs)
         if as_while:
             n_outs = n_outs - 1
         outs_info = [OrderedDict() for x in range(n_outs)]
@@ -864,7 +845,7 @@ def scan(
     shared_inner_inputs = []
     shared_inner_outputs = []
     sit_sot_shared = []
-    for input in dummy_f.maker.expanded_inputs:
+    for input in dummy_inputs:
         if isinstance(input.variable, SharedVariable) and input.update:
             new_var = safe_new(input.variable)
             if getattr(input.variable, "name", None) is not None:
@@ -873,11 +854,11 @@ def scan(
                 sit_sot_inner_inputs.append(new_var)
                 sit_sot_scan_inputs.append(
                     utils.expand_empty(
-                        aet.unbroadcast(shape_padleft(input.variable), 0),
+                        at.unbroadcast(shape_padleft(input.variable), 0),
                         actual_n_steps,
                     )
                 )
-                tensor_update = aet.as_tensor_variable(input.update)
+                tensor_update = at.as_tensor_variable(input.update)
                 sit_sot_inner_outputs.append(tensor_update)
                 # Note that `pos` is not a negative index. The sign of `pos` is used
                 # as a flag to indicate if this output should be part of the
@@ -898,6 +879,7 @@ def scan(
                 n_shared_outs += 1
 
     n_sit_sot = len(sit_sot_inner_inputs)
+
     # Step 5.4 Outputs with no taps used in the input
     n_nit_sot = 0
     nit_sot_inner_outputs = []
@@ -935,7 +917,7 @@ def scan(
 
         other_shared_scan_args = [
             arg.variable
-            for arg in dummy_f.maker.expanded_inputs
+            for arg in dummy_inputs
             if (
                 isinstance(arg.variable, SharedVariable)
                 and not arg.update
@@ -944,7 +926,7 @@ def scan(
         ]
         other_shared_inner_args = [
             safe_new(arg.variable, "_copy")
-            for arg in dummy_f.maker.expanded_inputs
+            for arg in dummy_inputs
             if (
                 isinstance(arg.variable, SharedVariable)
                 and not arg.update
@@ -954,12 +936,12 @@ def scan(
     else:
         other_shared_scan_args = [
             arg.variable
-            for arg in dummy_f.maker.expanded_inputs
+            for arg in dummy_inputs
             if (isinstance(arg.variable, SharedVariable) and not arg.update)
         ]
         other_shared_inner_args = [
             safe_new(arg.variable, "_copy")
-            for arg in dummy_f.maker.expanded_inputs
+            for arg in dummy_inputs
             if (isinstance(arg.variable, SharedVariable) and not arg.update)
         ]
     givens.update(OrderedDict(zip(other_shared_scan_args, other_shared_inner_args)))
@@ -1066,14 +1048,14 @@ def scan(
     scan_inputs = []
     for arg in [actual_n_steps] + _scan_inputs:
         try:
-            arg = aet.as_tensor_variable(arg)
+            arg = at.as_tensor_variable(arg)
         except TypeError:
             # This happens for Random States for e.g. but it is a good way
             # to make sure all inputs are tensors.
             pass
         scan_inputs += [arg]
     scan_outs = local_op(*scan_inputs)
-    if type(scan_outs) not in (list, tuple):
+    if not isinstance(scan_outs, (list, tuple)):
         scan_outs = [scan_outs]
     ##
     # Step 9. Figure out which outs are update rules for shared variables
@@ -1136,7 +1118,7 @@ def scan(
             # refers to update rule of index -1 - `pos`.
             update_map[sit_sot_shared[abs(pos) - 1]] = _scan_out_list[idx][-1]
     scan_out_list = [x for x in scan_out_list if x is not None]
-    if return_list is not True and len(scan_out_list) == 1:
+    if not return_list and len(scan_out_list) == 1:
         scan_out_list = scan_out_list[0]
     elif len(scan_out_list) == 0:
         scan_out_list = None
