@@ -187,7 +187,7 @@ class DimShuffle(ExternalCOp):
     def make_node(self, _input):
         input = as_tensor_variable(_input)
         ib = tuple(input.type.broadcastable)
-        if not ib == self.input_broadcastable:
+        if ib != self.input_broadcastable:
             if len(ib) != len(self.input_broadcastable):
                 raise TypeError(
                     "The number of dimensions of the "
@@ -209,7 +209,7 @@ class DimShuffle(ExternalCOp):
             else:
                 ob.append(ib[value])
 
-        output = TensorType(dtype=input.type.dtype, broadcastable=ob)()
+        output = TensorType(dtype=input.type.dtype, shape=ob)()
 
         return Apply(self, [input], [output])
 
@@ -223,7 +223,7 @@ class DimShuffle(ExternalCOp):
         (res,) = inp
         (storage,) = out
 
-        if type(res) != np.ndarray and type(res) != np.memmap:
+        if not isinstance(res, (np.ndarray, np.memmap)):
             raise TypeError(res)
 
         res = res.transpose(self.transposition)
@@ -303,40 +303,21 @@ pprint.assign(DimShuffle, DimShufflePrinter())
 
 
 class Elemwise(OpenMPOp):
-    """
-    Generalizes a scalar op to tensors.
+    """Generalizes a scalar `Op` to tensors.
 
     All the inputs must have the same number of dimensions. When the
-    Op is performed, for each dimension, each input's size for that
-    dimension must be the same. As a special case, it can also be 1
-    but only if the input's broadcastable flag is True for that
+    `Op` is performed, for each dimension, each input's size for that
+    dimension must be the same. As a special case, it can also be one
+    but only if the input's `broadcastable` flag is ``True`` for that
     dimension. In that case, the tensor is (virtually) replicated
     along that dimension to match the size of the others.
 
-    The dtypes of the outputs mirror those of the scalar Op that is
+    The dtypes of the outputs mirror those of the scalar `Op` that is
     being generalized to tensors. In particular, if the calculations
-    for an output are done inplace on an input, the output type must
+    for an output are done in-place on an input, the output type must
     be the same as the corresponding input type (see the doc of
     `ScalarOp` to get help about controlling the output type)
 
-    Parameters
-    ----------
-    scalar_op
-        An instance of a subclass of `ScalarOp` which works uniquely
-        on scalars.
-    inplace_pattern
-        A dictionary that maps the index of an output to the
-        index of an input so the output is calculated inplace using
-        the input's storage. (Just like destroymap, but without the lists.)
-    nfunc_spec
-        Either None or a tuple of three elements,
-        (nfunc_name, nin, nout) such that getattr(numpy, nfunc_name)
-        implements this operation, takes nin inputs and nout outputs.
-        Note that nin cannot always be inferred from the scalar op's
-        own nin field because that value is sometimes 0 (meaning a
-        variable number of inputs), whereas the numpy function may
-        not have varargs.
-
     Notes
     -----
     | Elemwise(add) represents + on tensors (x + y)
@@ -356,6 +337,26 @@ second dimension
     def __init__(
         self, scalar_op, inplace_pattern=None, name=None, nfunc_spec=None, openmp=None
     ):
+        """
+
+        Parameters
+        ----------
+        scalar_op
+            An instance of a subclass of `ScalarOp` which works uniquely
+            on scalars.
+        inplace_pattern
+            A dictionary that maps the index of an output to the
+            index of an input so the output is calculated inplace using
+            the input's storage. (Just like `Op.destroy_map`, but without the lists.)
+        nfunc_spec
+            Either ``None`` or a tuple of three elements, ``(nfunc_name, nin,
+            nout)`` such that ``getattr(numpy, nfunc_name)`` implements this
+            operation, takes ``nin``-many inputs and ``nout``-many outputs.  Note
+            that ``nin`` cannot always be inferred from the scalar `Op`'s own
+            ``nin`` field, because that value is sometimes zero (meaning a variable
+            number of inputs), whereas the NumPy function may not have var-args.
+
+        """
         assert not isinstance(scalar_op, type(self))
         if inplace_pattern is None:
             inplace_pattern = frozendict({})
@@ -463,7 +464,7 @@ second dimension
             DimShuffle, *inputs
         )
         outputs = [
-            TensorType(dtype=dtype, broadcastable=broadcastable)()
+            TensorType(dtype=dtype, shape=broadcastable)()
             for dtype, broadcastable in zip(out_dtypes, out_broadcastables)
         ]
         return Apply(self, inputs, outputs)
@@ -521,7 +522,7 @@ second dimension
         return [[True for output in node.outputs] for ipt in node.inputs]
 
     def L_op(self, inputs, outs, ograds):
-        from aesara.tensor.math import sum as aet_sum
+        from aesara.tensor.math import sum as at_sum
 
         # Compute grad with respect to broadcasted input
         rval = self._bgrad(inputs, outs, ograds)
@@ -562,7 +563,7 @@ second dimension
             ]
 
             if to_sum:
-                sr = aet_sum(rval[i], axis=to_sum, keepdims=True)
+                sr = at_sum(rval[i], axis=to_sum, keepdims=True)
                 rval[i] = sr
 
         return rval
@@ -849,8 +850,8 @@ second dimension
         # This is to protect again futur change of uniq.
         assert len(inames) == len(inputs)
         ii, iii = list(zip(*uniq(list(zip(_inames, node.inputs)))))
-        assert all([x == y for x, y in zip(ii, inames)])
-        assert all([x == y for x, y in zip(iii, inputs)])
+        assert all(x == y for x, y in zip(ii, inames))
+        assert all(x == y for x, y in zip(iii, inputs))
 
         defines = ""
         undefs = ""
@@ -1072,7 +1073,7 @@ second dimension
         # and the scalar op define optimized code for that case
         # use it! The scalar_op need to check the broadcast flag himself.
         if (
-            all([o.ndim >= 1 for o in node.outputs])
+            all(o.ndim >= 1 for o in node.outputs)
             and
             # Don't use the contig code for broadcasted scalar.
             not all(node.outputs[0].broadcastable)
@@ -1212,24 +1213,15 @@ second dimension
 
 
 class CAReduce(COp):
-    """
-    CAReduce = Commutative Associative Reduce
-    Reduces a scalar operation along the specified axis(es).
-    (The scalar op should be both commutative and assocative)
+    """Reduces a scalar operation along specified axes.
+
+    The scalar op should be both commutative and associative.
+
+    `CAReduce` = Commutative Associative Reduce.
 
     The output will have the same shape as the input minus the reduced
     dimensions. It will contain the variable of accumulating all values
-    over the reduced dimensions using the specified scalar op.
-
-    Parameters
-    ----------
-    scalar_op
-        A binary scalar op with only one output.
-        It must be commutative and associative.
-    axis
-        - The dimension along which we want to reduce
-        - List of dimensions that we want to reduce
-        - If None, all dimensions are reduced
+    over the reduced dimensions using the specified scalar `Op`.
 
     Notes
     -----
@@ -1246,7 +1238,7 @@ class CAReduce(COp):
                            # even number ...
 
     In order to (eventually) optimize memory usage patterns,
-    CAReduce makes zero guarantees on the order in which it
+    `CAReduce` makes zero guarantees on the order in which it
     iterates over the dimensions and the elements of the
     array(s). Therefore, to ensure consistent variables, the scalar
     operation represented by the reduction must be both commutative
@@ -1260,7 +1252,20 @@ class CAReduce(COp):
     ] = ("scalar_op", "axis")
 
     def __init__(self, scalar_op, axis=None):
-        if scalar_op.nin not in [-1, 2] or scalar_op.nout != 1:
+        """
+
+        Parameters
+        ----------
+        scalar_op
+            A binary scalar `Op` with only one output. It must be commutative
+            and associative.
+        axis
+            - The dimension along which we want to reduce
+            - List of dimensions that we want to reduce
+            - If ``None``, all dimensions are reduced
+
+        """
+        if scalar_op.nin not in (-1, 2) or scalar_op.nout != 1:
             raise NotImplementedError(
                 "CAReduce only supports binary functions with a single " "output."
             )
@@ -1321,9 +1326,7 @@ class CAReduce(COp):
 
         broadcastable = [x for i, x in enumerate(inp_bdcast) if i not in axis]
 
-        output = TensorType(
-            dtype=self._output_dtype(inp_dtype), broadcastable=broadcastable
-        )()
+        output = TensorType(dtype=self._output_dtype(inp_dtype), shape=broadcastable)()
 
         return Apply(op, [input], [output])
 
@@ -1404,9 +1407,7 @@ class CAReduce(COp):
         if acc_dtype is not None:
             if acc_dtype == "float16":
                 raise MethodNotDefined("no c_code for float16")
-            acc_type = TensorType(
-                broadcastable=node.outputs[0].broadcastable, dtype=acc_dtype
-            )
+            acc_type = TensorType(shape=node.outputs[0].broadcastable, dtype=acc_dtype)
             adtype = acc_type.dtype_specs()[1]
         else:
             adtype = odtype
@@ -1480,14 +1481,14 @@ class CAReduce(COp):
         identity = self.scalar_op.identity
 
         if np.isposinf(identity):
-            if input.type.dtype in ["float32", "float64"]:
+            if input.type.dtype in ("float32", "float64"):
                 identity = "__builtin_inf()"
             elif input.type.dtype.startswith("uint") or input.type.dtype == "bool":
                 identity = "1"
             else:
                 identity = "NPY_MAX_" + str(input.type.dtype).upper()
         elif np.isneginf(identity):
-            if input.type.dtype in ["float32", "float64"]:
+            if input.type.dtype in ("float32", "float64"):
                 identity = "-__builtin_inf()"
             elif input.type.dtype.startswith("uint") or input.type.dtype == "bool":
                 identity = "0"
@@ -1588,56 +1589,15 @@ class CAReduce(COp):
 
 
 class CAReduceDtype(CAReduce):
-    """
-    Reduces a scalar operation along the specified axis(es).
+    """A subclass of `CAReduce` that accepts an additional output "dtype" parameter.
 
-    This subclass of CAReduce accepts an additional "dtype" parameter,
-    that specifies which dtype the output should be.
+    It also accepts an optional `acc_dtype`, which specifies the dtype that
+    will be used for the accumulation.  The accumulation will be done using an
+    array of dtype `acc_dtype`, then it will be cast into `dtype` and returned.
 
-    It also accepts an optional "acc_dtype", which specify the dtype that
-    will be used for the accumulation.
-
-    So, the accumulation will be done into a tensor of dtype "acc_dtype",
-    then it will be casted into "dtype" and returned.
-
-    If no dtype is provided, one will be inferred so as not to lose
+    If no `dtype` is provided, one will be inferred so as not to lose
     too much precision.
 
-    Parameters
-    ----------
-    scalar_op
-        A binary scalar op with only one output.
-        It must be commutative and associative.
-
-    axis
-        * the dimension along which we want to reduce
-        * list of dimensions that we want to reduce
-        * if None, all dimensions are reduced
-
-    dtype
-        The dtype of the returned tensor. If None, then we use the default
-        dtype which is the same as the input tensor's dtype except when:
-
-        * the input dtype is a signed integer of precision < 64 bit, in which
-          case we use int64
-        * the input dtype is an unsigned integer of precision < 64 bit, in
-          which case we use uint64
-
-        This default dtype does _not_ depend on the value of "acc_dtype".
-        This behavior is similar in spirit to that of numpy (except numpy
-        uses the default machine integer while we always use 64 bit
-        integers to avoid platform-dependent behavior).
-
-    acc_dtype
-        The dtype of the internal accumulator.
-        If None (default), we use the dtype in the list below,
-        or the input dtype if its precision is higher:
-
-        * for int dtypes, we use at least int64;
-        * for uint dtypes, we use at least uint64;
-        * for float dtypes, we use at least float64;
-        * for complex dtypes, we use at least complex128.
-
     """
 
     __props__: Union[Tuple[str, str, str], Tuple[str, str, str, str]] = (
@@ -1648,6 +1608,41 @@ class CAReduceDtype(CAReduce):
     )
 
     def __init__(self, scalar_op, axis=None, dtype=None, acc_dtype=None):
+        """
+
+        Parameters
+        ----------
+        scalar_op
+            A binary scalar `Op` with only one output.
+            It must be commutative and associative.
+        axis
+            * the dimension along which we want to reduce
+            * list of dimensions that we want to reduce
+            * if ``None``, all dimensions are reduced
+        dtype
+            The dtype of the returned tensor. If ``None``, then we use the default
+            dtype which is the same as the input array's dtype except when:
+
+            * the input dtype is a signed integer of precision < 64 bit, in which
+            case we use int64
+            * the input dtype is an unsigned integer of precision < 64 bit, in
+            which case we use uint64
+
+            This default dtype does _not_ depend on the value of `acc_dtype`.
+            This behavior is similar in spirit to that of NumPy, except that
+            NumPy uses the default machine integer while we always use 64 bit
+            integers to avoid platform-dependent behavior.
+        acc_dtype
+            The dtype of the internal accumulator.
+            If ``None`` (default), we use the dtype in the list below,
+            or the input dtype if its precision is higher:
+
+            * for int dtypes, we use at least int64;
+            * for uint dtypes, we use at least uint64;
+            * for float dtypes, we use at least float64;
+            * for complex dtypes, we use at least complex128.
+
+        """
         super().__init__(scalar_op, axis=axis)
         self.dtype = dtype
         self.acc_dtype = acc_dtype
