@@ -1474,8 +1474,11 @@ def complex_from_polar(abs, angle):
 
 
 class Mean(CAReduce):
+    __props__ = ("axis",)
+    nfunc_spec = ("mean", 1, 1)
+
     def __init__(self, axis=None):
-        super().__init__(aes.add, axis)
+        super().__init__(aes.mean, axis)
         assert self.axis is None or len(self.axis) == 1
 
     def __str__(self):
@@ -2563,7 +2566,7 @@ class Prod(CAReduceDtype):
         one zero.
 
         I do this by first counting the number of zeros in each group (see the
-        `aet.eq` bits), then taking this or that behavior (see `aet.switch`)
+        `at.eq` bits), then taking this or that behavior (see `at.switch`)
         based on the result of this count.
 
         """
@@ -2598,7 +2601,7 @@ class Prod(CAReduceDtype):
         gz = gz.dimshuffle(new_dims)
 
         # division trick if we don't have zeros. This will contain
-        # NaNs to be eliminated in the `aet.switch` if we do have zeros.
+        # NaNs to be eliminated in the `at.switch` if we do have zeros.
         grad_case_without_zeros = gz * prod_out / prod_in
 
         if self.no_zeros_in_input:
