@@ -1,3 +1,4 @@
+import abc
 from typing import List, Optional, Union
 
 import numpy as np
@@ -6,15 +7,58 @@ import scipy.stats as stats
 import aesara
 from aesara.tensor.basic import as_tensor_variable
 from aesara.tensor.random.op import RandomVariable, default_shape_from_params
+from aesara.tensor.random.type import RandomGeneratorType, RandomStateType
 from aesara.tensor.random.utils import broadcast_params
+from aesara.tensor.random.var import (
+    RandomGeneratorSharedVariable,
+    RandomStateSharedVariable,
+)
 
 
 try:
-    from pypolyagamma import PyPolyaGamma
-except ImportError:  # pragma: no cover
+    broadcast_shapes = np.broadcast_shapes
+except AttributeError:
+    from numpy.lib.stride_tricks import _broadcast_shape
 
-    def PyPolyaGamma(*args, **kwargs):
-        raise RuntimeError("pypolygamma not installed!")
+    def broadcast_shapes(*shapes):
+        return _broadcast_shape(*[np.empty(x, dtype=[]) for x in shapes])
+
+
+class ScipyRandomVariable(RandomVariable):
+    r"""A class for `RandomVariable`\s that use SciPy-based samplers.
+
+    This will only work for `RandomVariable`\s for which the output shape is
+    entirely determined by broadcasting the distribution parameters (e.g. basic
+    scalar distributions).
+
+    The more sophisticated shape logic performed by `RandomVariable` is avoided
+    in order to reduce the amount of unnecessary extra steps taken to correct
+    SciPy's shape-removing defect.
+
+    """
+
+    @classmethod
+    @abc.abstractmethod
+    def rng_fn_scipy(cls, rng, *args, **kwargs):
+        r"""
+
+        `RandomVariable`\s implementations that want to use SciPy-based samplers
+        need to implement this method instead of the base
+        `RandomVariable.rng_fn`; otherwise their broadcast dimensions will be
+        dropped by SciPy.
+
+        """
+
+    @classmethod
+    def rng_fn(cls, *args, **kwargs):
+        size = args[-1]
+        res = cls.rng_fn_scipy(*args, **kwargs)
+        return np.broadcast_to(
+            res,
+            size
+            if size is not None
+            else broadcast_shapes(*[np.shape(a) for a in args[1:-1]]),
+        )
 
 
 class UniformRV(RandomVariable):
@@ -67,7 +111,15 @@ class NormalRV(RandomVariable):
 normal = NormalRV()
 
 
-class HalfNormalRV(RandomVariable):
+class StandardNormalRV(NormalRV):
+    def __call__(self, size=None, **kwargs):
+        return super().__call__(loc=0.0, scale=1.0, size=size, **kwargs)
+
+
+standard_normal = StandardNormalRV()
+
+
+class HalfNormalRV(ScipyRandomVariable):
     name = "halfnormal"
     ndim_supp = 0
     ndims_params = [0, 0]
@@ -78,7 +130,7 @@ class HalfNormalRV(RandomVariable):
         return super().__call__(loc, scale, size=size, **kwargs)
 
     @classmethod
-    def rng_fn(cls, rng, loc, scale, size):
+    def rng_fn_scipy(cls, rng, loc, scale, size):
         return stats.halfnorm.rvs(loc, scale, random_state=rng, size=size)
 
 
@@ -99,7 +151,7 @@ class LogNormalRV(RandomVariable):
 lognormal = LogNormalRV()
 
 
-class GammaRV(RandomVariable):
+class GammaRV(ScipyRandomVariable):
     name = "gamma"
     ndim_supp = 0
     ndims_params = [0, 0]
@@ -110,7 +162,7 @@ class GammaRV(RandomVariable):
         return super().__call__(shape, 1.0 / rate, size=size, **kwargs)
 
     @classmethod
-    def rng_fn(cls, rng, shape, scale, size):
+    def rng_fn_scipy(cls, rng, shape, scale, size):
         return stats.gamma.rvs(shape, scale=scale, size=size, random_state=rng)
 
 
@@ -128,7 +180,7 @@ class ChiSquareRV(RandomVariable):
 chisquare = ChiSquareRV()
 
 
-class ParetoRV(RandomVariable):
+class ParetoRV(ScipyRandomVariable):
     name = "pareto"
     ndim_supp = 0
     ndims_params = [0, 0]
@@ -139,14 +191,14 @@ class ParetoRV(RandomVariable):
         return super().__call__(b, scale, size=size, **kwargs)
 
     @classmethod
-    def rng_fn(cls, rng, b, scale, size):
+    def rng_fn_scipy(cls, rng, b, scale, size):
         return stats.pareto.rvs(b, scale=scale, size=size, random_state=rng)
 
 
 pareto = ParetoRV()
 
 
-class GumbelRV(RandomVariable):
+class GumbelRV(ScipyRandomVariable):
     name = "gumbel"
     ndim_supp = 0
     ndims_params = [0, 0]
@@ -158,14 +210,14 @@ class GumbelRV(RandomVariable):
         loc: Union[np.ndarray, float],
         scale: Union[np.ndarray, float] = 1.0,
         size: Optional[Union[List[int], int]] = None,
-        **kwargs
+        **kwargs,
     ) -> RandomVariable:
         return super().__call__(loc, scale, size=size, **kwargs)
 
     @classmethod
-    def rng_fn(
+    def rng_fn_scipy(
         cls,
-        rng: np.random.RandomState,
+        rng: Union[np.random.Generator, np.random.RandomState],
         loc: Union[np.ndarray, float],
         scale: Union[np.ndarray, float],
         size: Optional[Union[List[int], int]],
@@ -269,14 +321,21 @@ class MvNormalRV(RandomVariable):
 
         if mean.ndim > 1 or cov.ndim > 2:
             # Neither SciPy nor NumPy implement parameter broadcasting for
-            # multivariate normals (or many other multivariate distributions),
-            # so we have implement a quick and dirty one here
+            # multivariate normals (or any other multivariate distributions),
+            # so we need to implement that here
             mean, cov = broadcast_params([mean, cov], cls.ndims_params)
             size = tuple(size or ())
 
             if size:
-                mean = np.broadcast_to(mean, size + mean.shape)
-                cov = np.broadcast_to(cov, size + cov.shape)
+                if (
+                    0 < mean.ndim - 1 <= len(size)
+                    and size[-mean.ndim + 1 :] != mean.shape[:-1]
+                ):
+                    raise ValueError(
+                        "shape mismatch: objects cannot be broadcast to a single shape"
+                    )
+                mean = np.broadcast_to(mean, size + mean.shape[-1:])
+                cov = np.broadcast_to(cov, size + cov.shape[-2:])
 
             res = np.empty(mean.shape)
             for idx in np.ndindex(mean.shape[:-1]):
@@ -300,16 +359,33 @@ class DirichletRV(RandomVariable):
 
     @classmethod
     def rng_fn(cls, rng, alphas, size):
-        if size is None:
-            size = ()
-        samples_shape = tuple(np.atleast_1d(size)) + alphas.shape
-        samples = np.empty(samples_shape)
-        alphas_bcast = np.broadcast_to(alphas, samples_shape)
+        if alphas.ndim > 1:
+            if size is None:
+                size = ()
 
-        for index in np.ndindex(*samples_shape[:-1]):
-            samples[index] = rng.dirichlet(alphas_bcast[index])
+            size = tuple(np.atleast_1d(size))
 
-        return samples
+            if size:
+                if (
+                    0 < alphas.ndim - 1 <= len(size)
+                    and size[-alphas.ndim + 1 :] != alphas.shape[:-1]
+                ):
+                    raise ValueError(
+                        "shape mismatch: objects cannot be broadcast to a single shape"
+                    )
+                samples_shape = size + alphas.shape[-1:]
+            else:
+                samples_shape = alphas.shape
+
+            samples = np.empty(samples_shape)
+            alphas_bcast = np.broadcast_to(alphas, samples_shape)
+
+            for index in np.ndindex(*samples_shape[:-1]):
+                samples[index] = rng.dirichlet(alphas_bcast[index])
+
+            return samples
+        else:
+            return rng.dirichlet(alphas, size=size)
 
 
 dirichlet = DirichletRV()
@@ -351,7 +427,7 @@ class HyperGeometricRV(RandomVariable):
 hypergeometric = HyperGeometricRV()
 
 
-class CauchyRV(RandomVariable):
+class CauchyRV(ScipyRandomVariable):
     name = "cauchy"
     ndim_supp = 0
     ndims_params = [0, 0]
@@ -362,14 +438,14 @@ class CauchyRV(RandomVariable):
         return super().__call__(loc, scale, size=size, **kwargs)
 
     @classmethod
-    def rng_fn(cls, rng, loc, scale, size):
+    def rng_fn_scipy(cls, rng, loc, scale, size):
         return stats.cauchy.rvs(loc=loc, scale=scale, random_state=rng, size=size)
 
 
 cauchy = CauchyRV()
 
 
-class HalfCauchyRV(RandomVariable):
+class HalfCauchyRV(ScipyRandomVariable):
     name = "halfcauchy"
     ndim_supp = 0
     ndims_params = [0, 0]
@@ -380,14 +456,14 @@ class HalfCauchyRV(RandomVariable):
         return super().__call__(loc, scale, size=size, **kwargs)
 
     @classmethod
-    def rng_fn(cls, rng, loc, scale, size):
+    def rng_fn_scipy(cls, rng, loc, scale, size):
         return stats.halfcauchy.rvs(loc=loc, scale=scale, random_state=rng, size=size)
 
 
 halfcauchy = HalfCauchyRV()
 
 
-class InvGammaRV(RandomVariable):
+class InvGammaRV(ScipyRandomVariable):
     name = "invgamma"
     ndim_supp = 0
     ndims_params = [0, 0]
@@ -395,7 +471,7 @@ class InvGammaRV(RandomVariable):
     _print_name = ("InvGamma", "\\operatorname{Gamma^{-1}}")
 
     @classmethod
-    def rng_fn(cls, rng, shape, rate, size=None):
+    def rng_fn_scipy(cls, rng, shape, rate, size):
         return stats.invgamma.rvs(shape, scale=rate, size=size, random_state=rng)
 
 
@@ -416,7 +492,7 @@ class WaldRV(RandomVariable):
 wald = WaldRV()
 
 
-class TruncExponentialRV(RandomVariable):
+class TruncExponentialRV(ScipyRandomVariable):
     name = "truncexpon"
     ndim_supp = 0
     ndims_params = [0, 0, 0]
@@ -424,7 +500,7 @@ class TruncExponentialRV(RandomVariable):
     _print_name = ("TruncExp", "\\operatorname{TruncExp}")
 
     @classmethod
-    def rng_fn(cls, rng, b, loc, scale, size=None):
+    def rng_fn_scipy(cls, rng, b, loc, scale, size):
         return stats.truncexpon.rvs(
             b, loc=loc, scale=scale, size=size, random_state=rng
         )
@@ -433,7 +509,7 @@ class TruncExponentialRV(RandomVariable):
 truncexpon = TruncExponentialRV()
 
 
-class BernoulliRV(RandomVariable):
+class BernoulliRV(ScipyRandomVariable):
     name = "bernoulli"
     ndim_supp = 0
     ndims_params = [0]
@@ -441,7 +517,7 @@ class BernoulliRV(RandomVariable):
     _print_name = ("Bern", "\\operatorname{Bern}")
 
     @classmethod
-    def rng_fn(cls, rng, p, size=None):
+    def rng_fn_scipy(cls, rng, p, size):
         return stats.bernoulli.rvs(p, size=size, random_state=rng)
 
 
@@ -470,7 +546,7 @@ class BinomialRV(RandomVariable):
 binomial = BinomialRV()
 
 
-class NegBinomialRV(RandomVariable):
+class NegBinomialRV(ScipyRandomVariable):
     name = "nbinom"
     ndim_supp = 0
     ndims_params = [0, 0]
@@ -478,14 +554,14 @@ class NegBinomialRV(RandomVariable):
     _print_name = ("NB", "\\operatorname{NB}")
 
     @classmethod
-    def rng_fn(cls, rng, n, p, size=None):
+    def rng_fn_scipy(cls, rng, n, p, size):
         return stats.nbinom.rvs(n, p, size=size, random_state=rng)
 
 
 nbinom = NegBinomialRV()
 
 
-class BetaBinomialRV(RandomVariable):
+class BetaBinomialRV(ScipyRandomVariable):
     name = "beta_binomial"
     ndim_supp = 0
     ndims_params = [0, 0, 0]
@@ -493,7 +569,7 @@ class BetaBinomialRV(RandomVariable):
     _print_name = ("BetaBinom", "\\operatorname{BetaBinom}")
 
     @classmethod
-    def rng_fn(cls, rng, n, a, b, size=None):
+    def rng_fn_scipy(cls, rng, n, a, b, size):
         return stats.betabinom.rvs(n, a, b, size=size, random_state=rng)
 
 
@@ -527,10 +603,14 @@ class MultinomialRV(RandomVariable):
             size = tuple(size or ())
 
             if size:
-                n = np.broadcast_to(n, size + n.shape)
-                p = np.broadcast_to(p, size + p.shape)
+                if 0 < p.ndim - 1 <= len(size) and size[-p.ndim + 1 :] != p.shape[:-1]:
+                    raise ValueError(
+                        "shape mismatch: objects cannot be broadcast to a single shape"
+                    )
+                n = np.broadcast_to(n, size)
+                p = np.broadcast_to(p, size + p.shape[-1:])
 
-            res = np.empty(p.shape)
+            res = np.empty(p.shape, dtype=cls.dtype)
             for idx in np.ndindex(p.shape[:-1]):
                 res[idx] = rng.multinomial(n[idx], p[idx])
             return res
@@ -575,46 +655,6 @@ class CategoricalRV(RandomVariable):
 categorical = CategoricalRV()
 
 
-class PolyaGammaRV(RandomVariable):
-    """Polya-Gamma random variable.
-
-    XXX: This doesn't really use the given RNG, due to the narrowness of the
-    sampler package's implementation.
-    """
-
-    name = "polya-gamma"
-    ndim_supp = 0
-    ndims_params = [0, 0]
-    dtype = "floatX"
-    _print_name = ("PG", "\\operatorname{PG}")
-
-    @classmethod
-    def rng_fn(cls, rng, b, c, size):
-        pg = PyPolyaGamma(rng.randint(2 ** 16))
-
-        if not size and b.shape == c.shape == ():
-            return pg.pgdraw(b, c)
-        else:
-            b, c = np.broadcast_arrays(b, c)
-            size = tuple(size or ())
-
-            if len(size) > 0:
-                b = np.broadcast_to(b, size)
-                c = np.broadcast_to(c, size)
-
-            smpl_val = np.empty(b.shape, dtype="double")
-
-            pg.pgdrawv(
-                np.asarray(b.flat).astype("double", copy=True),
-                np.asarray(c.flat).astype("double", copy=True),
-                np.asarray(smpl_val.flat),
-            )
-            return smpl_val
-
-
-polyagamma = PolyaGammaRV()
-
-
 class RandIntRV(RandomVariable):
     name = "randint"
     ndim_supp = 0
@@ -627,10 +667,41 @@ class RandIntRV(RandomVariable):
             low, high = 0, low
         return super().__call__(low, high, size=size, **kwargs)
 
+    def make_node(self, rng, *args, **kwargs):
+        if not isinstance(
+            getattr(rng, "type", None), (RandomStateType, RandomStateSharedVariable)
+        ):
+            raise TypeError("`randint` is only available for `RandomStateType`s")
+        return super().make_node(rng, *args, **kwargs)
+
 
 randint = RandIntRV()
 
 
+class IntegersRV(RandomVariable):
+    name = "integers"
+    ndim_supp = 0
+    ndims_params = [0, 0]
+    dtype = "int64"
+    _print_name = ("integers", "\\operatorname{integers}")
+
+    def __call__(self, low, high=None, size=None, **kwargs):
+        if high is None:
+            low, high = 0, low
+        return super().__call__(low, high, size=size, **kwargs)
+
+    def make_node(self, rng, *args, **kwargs):
+        if not isinstance(
+            getattr(rng, "type", None),
+            (RandomGeneratorType, RandomGeneratorSharedVariable),
+        ):
+            raise TypeError("`integers` is only available for `RandomGeneratorType`s")
+        return super().make_node(rng, *args, **kwargs)
+
+
+integers = IntegersRV()
+
+
 class ChoiceRV(RandomVariable):
     name = "choice"
     ndim_supp = 0
@@ -698,6 +769,7 @@ permutation = PermutationRV()
 __all__ = [
     "permutation",
     "choice",
+    "integers",
     "randint",
     "categorical",
     "multinomial",
@@ -730,4 +802,5 @@ __all__ = [
     "beta",
     "triangular",
     "uniform",
+    "standard_normal",
 ]
