@@ -17,10 +17,11 @@ from aesara.tensor.basic import (
     get_vector_length,
     infer_broadcastable,
 )
-from aesara.tensor.random.type import RandomType
+from aesara.tensor.random.type import RandomGeneratorType, RandomStateType, RandomType
 from aesara.tensor.random.utils import normalize_size_param, params_broadcast_shapes
 from aesara.tensor.shape import shape_tuple
 from aesara.tensor.type import TensorType, all_dtypes
+from aesara.tensor.type_other import NoneConst
 from aesara.tensor.var import TensorVariable
 
 
@@ -189,15 +190,14 @@ class RandomVariable(Op):
 
         size_len = get_vector_length(size)
 
-        if self.ndim_supp == 0 and size_len > 0:
-            # In this case, we have a univariate distribution with a non-empty
-            # `size` parameter, which means that the `size` parameter
-            # completely determines the shape of the random variable.  More
-            # importantly, the `size` parameter may be the only correct source
-            # of information for the output shape, in that we would be misled
-            # by the `dist_params` if we tried to infer the relevant parts of
-            # the output shape from those.
-            return size
+        if size_len > 0:
+            if self.ndim_supp == 0:
+                return size
+            else:
+                supp_shape = self._shape_from_params(
+                    dist_params, param_shapes=param_shapes
+                )
+                return tuple(size) + tuple(supp_shape)
 
         # Broadcast the parameters
         param_shapes = params_broadcast_shapes(
@@ -306,19 +306,19 @@ class RandomVariable(Op):
             Existing Aesara `Generator` or `RandomState` object to be used.  Creates a
             new one, if `None`.
         size: int or Sequence
-            Numpy-like size of the output (i.e. replications).
+            NumPy-like size parameter.
         dtype: str
             The dtype of the sampled output.  If the value ``"floatX"`` is
-            given, then ``dtype`` is set to ``aesara.config.floatX``.  This
-            value is only used when `self.dtype` isn't set.
+            given, then `dtype` is set to ``aesara.config.floatX``.  This value is
+            only used when ``self.dtype`` isn't set.
         dist_params: list
             Distribution parameters.
 
         Results
         -------
-        out: `Apply`
-            A node with inputs `(rng, size, dtype) + dist_args` and outputs
-            `(rng_var, out_var)`.
+        out: Apply
+            A node with inputs ``(rng, size, dtype) + dist_args`` and outputs
+            ``(rng_var, out_var)``.
 
         """
         size = normalize_size_param(size)
@@ -350,7 +350,7 @@ class RandomVariable(Op):
             dtype_idx = constant(dtype, dtype="int64")
             dtype = all_dtypes[dtype_idx.data]
 
-        outtype = TensorType(dtype=dtype, broadcastable=bcast)
+        outtype = TensorType(dtype=dtype, shape=bcast)
         out_var = outtype()
         inputs = (rng, size, dtype_idx) + dist_params
         outputs = (rng.type(), out_var)
@@ -399,3 +399,36 @@ class RandomVariable(Op):
 
     def R_op(self, inputs, eval_points):
         return [None for i in eval_points]
+
+
+class AbstractRNGConstructor(Op):
+    def make_node(self, seed=None):
+        if seed is None:
+            seed = NoneConst
+        else:
+            seed = as_tensor_variable(seed)
+        inputs = [seed]
+        outputs = [self.random_type()]
+        return Apply(self, inputs, outputs)
+
+    def perform(self, node, inputs, output_storage):
+        (seed,) = inputs
+        if seed is not None and seed.size == 1:
+            seed = int(seed)
+        output_storage[0][0] = getattr(np.random, self.random_constructor)(seed=seed)
+
+
+class RandomStateConstructor(AbstractRNGConstructor):
+    random_type = RandomStateType()
+    random_constructor = "RandomState"
+
+
+RandomState = RandomStateConstructor()
+
+
+class DefaultGeneratorMakerOp(AbstractRNGConstructor):
+    random_type = RandomGeneratorType()
+    random_constructor = "default_rng"
+
+
+default_rng = DefaultGeneratorMakerOp()
