@@ -5,7 +5,7 @@ from aesara.graph.opt import in2out, local_optimizer
 from aesara.tensor.basic import constant, get_vector_length
 from aesara.tensor.elemwise import DimShuffle
 from aesara.tensor.extra_ops import broadcast_to
-from aesara.tensor.math import sum as aet_sum
+from aesara.tensor.math import sum as at_sum
 from aesara.tensor.random.op import RandomVariable
 from aesara.tensor.random.utils import broadcast_params
 from aesara.tensor.shape import Shape, Shape_i
@@ -83,9 +83,19 @@ def local_rv_size_lift(fgraph, node):
     if get_vector_length(size) > 0:
         dist_params = [
             broadcast_to(
-                p, (tuple(size) + tuple(p.shape)) if node.op.ndim_supp > 0 else size
+                p,
+                (
+                    tuple(size)
+                    + (
+                        tuple(p.shape)[-node.op.ndims_params[i] :]
+                        if node.op.ndims_params[i] > 0
+                        else ()
+                    )
+                )
+                if node.op.ndim_supp > 0
+                else size,
             )
-            for p in dist_params
+            for i, p in enumerate(dist_params)
         ]
     else:
         return
@@ -383,7 +393,7 @@ def local_subtensor_rv_lift(fgraph, node):
     # *which* dimensions of `size` are used).
     if any(st_is_bool):
         size_lifted = tuple(
-            aet_sum(idx) if is_bool else s
+            at_sum(idx) if is_bool else s
             for s, is_bool, idx in zip(
                 size_lifted, st_is_bool, st_indices[: (reps_ind_split_idx + 1)]
             )
