@@ -697,15 +697,13 @@ class Subtensor(COp):
         input_types = get_slice_elements(
             idx_list, lambda entry: isinstance(entry, Type)
         )
-        if len(inputs) != len(input_types):
-            raise IndexError(
-                "Not enough inputs to fill in the Subtensor template.", inputs, idx_list
-            )
+
+        assert len(inputs) == len(input_types)
+
         for input, expected_type in zip(inputs, input_types):
-            if input.type != expected_type:
+            if not expected_type.is_super(input.type):
                 raise TypeError(
-                    "Wrong type for Subtensor template. Expected %s, got %s."
-                    % (input.type, expected_type)
+                    f"Incompatible types for Subtensor template. Expected {input.type}, got {expected_type}."
                 )
 
         # infer the broadcasting pattern
@@ -737,7 +735,7 @@ class Subtensor(COp):
         return Apply(
             self,
             (x,) + inputs,
-            [tensor(dtype=x.type.dtype, broadcastable=broadcastable)],
+            [tensor(dtype=x.type.dtype, shape=broadcastable)],
         )
 
     def perform(self, node, inputs, out_):
@@ -1531,10 +1529,9 @@ class IncSubtensor(COp):
                 "Not enough inputs to fill in the Subtensor template.", inputs, idx_list
             )
         for input, expected_type in zip(inputs, input_types):
-            if input.type != expected_type:
+            if not expected_type.is_super(input.type):
                 raise TypeError(
-                    "Wrong type for Subtensor template. Expected %s, got %s."
-                    % (input.type, expected_type)
+                    f"Wrong type for Subtensor template. Expected {input.type}, got {expected_type}."
                 )
 
         return Apply(self, (x, y) + inputs, [x.type()])
@@ -1940,9 +1937,7 @@ class AdvancedSubtensor1(COp):
         if x_.type.ndim == 0:
             raise TypeError("cannot index into a scalar")
         bcast = (ilist_.broadcastable[0],) + x_.broadcastable[1:]
-        return Apply(
-            self, [x_, ilist_], [TensorType(dtype=x.dtype, broadcastable=bcast)()]
-        )
+        return Apply(self, [x_, ilist_], [TensorType(dtype=x.dtype, shape=bcast)()])
 
     def perform(self, node, inp, out_):
         x, i = inp
@@ -2218,7 +2213,7 @@ class AdvancedIncSubtensor1(COp):
             ]
         ]
 
-        complex_types = ["npy_" + t for t in ["complex32", "complex64", "complex128"]]
+        complex_types = ["npy_" + t for t in ("complex32", "complex64", "complex128")]
 
         inplace_map_template = """
         #if defined(%(typen)s)
@@ -2550,7 +2545,7 @@ class AdvancedSubtensor(Op):
         # entire subtensor operation.
         with config.change_flags(compute_test_value="off"):
             fake_shape = tuple(
-                tensor(dtype="int64", broadcastable=()) if not bcast else 1
+                tensor(dtype="int64", shape=()) if not bcast else 1
                 for bcast in x.broadcastable
             )
 
@@ -2572,7 +2567,7 @@ class AdvancedSubtensor(Op):
         return Apply(
             self,
             (x,) + index,
-            [tensor(dtype=x.type.dtype, broadcastable=bcast)],
+            [tensor(dtype=x.type.dtype, shape=bcast)],
         )
 
     def R_op(self, inputs, eval_points):
@@ -2675,7 +2670,7 @@ class AdvancedIncSubtensor(Op):
         return Apply(
             self,
             (x, y) + tuple(new_inputs),
-            [tensor(dtype=x.type.dtype, broadcastable=x.type.broadcastable)],
+            [tensor(dtype=x.type.dtype, shape=x.type.broadcastable)],
         )
 
     def perform(self, node, inputs, out_):
@@ -2775,10 +2770,8 @@ def take(a, indices, axis=None, mode="raise"):
     if not isinstance(axis, (int, type(None))):
         raise TypeError("`axis` must be an integer or None")
 
-    if axis is None and indices.ndim == 1:
-        return advanced_subtensor1(a.flatten(), indices)
-    elif axis == 0 and indices.ndim == 1:
-        return advanced_subtensor1(a, indices)
+    if axis is None:
+        return advanced_subtensor(a.flatten(), indices)
     elif axis < 0:
         axis += a.ndim
 
